'*************************************************************************
' Klass: modMain
' Beroenden: 
' Syfte: 
' F�rfattare: Fredrik Haglund, Stefan Haglund
' Senast �ndrad av: Stefan Haglund
' Senast �ndrad: 2007-04-26 08:38
' Kommentarer: 
'*************************************************************************

Imports Microsoft.Win32
Imports System.IO
Imports CDBurnerXP.IO
Imports CDBurnerXP.Burning
Imports System.Threading
Imports System.Diagnostics
Imports CDBurnerXP.Net
Imports CDBurnerXP
Imports Microsoft.Win32.Windows7
Imports System.Globalization
Imports CDBurnerXP.Controls
Imports CDBurnerXP.Windows.Forms
Imports System.Resources
Imports CDBurnerXP.Forms

Module modMain
  Public g_sAction As String = ""
  Public g_sActionFile As String = ""

  Public Sub Main(ByVal args As String())

    ' Initialise settings
    Settings.Provider = New RegistrySettingsProvider("Software\Canneverbe Limited\CDBurnerXP")

    Try
      Try
        ApplicationSettings.Load()
      Catch ex As Exception
        CDBurnerXP.IO.Debug.LogError(ex)
      End Try

      ' Run the updater if necessary
      Dim argumentList As New List(Of String)(args)

      Dim updateEnabled As Boolean = AutoUpdate.IsOnlineUpdateAllowed AndAlso ApplicationSettings.AutoUpdate

      Dim updateSetup As String = AutoUpdate.GetLatestUpdate()
      If Not argumentList.Contains("/UpdateDone") AndAlso updateEnabled AndAlso Not String.IsNullOrEmpty(updateSetup) Then
        If File.Exists("Updater.exe") Then
          Dim fVerInfo As FileVersionInfo = FileVersionInfo.GetVersionInfo(updateSetup)
          Dim updateFileVersion As String = fVerInfo.FileVersion.Trim()

          ' Don't show message if CDBurnerXP has already been updated after the setup file has been downloaded
          If New Version(updateFileVersion) > New Version(Application.ProductVersion) Then
            ' Ask user to update to prevent surprises.
            If MessageBox.Show(BaseForm.FormatTranslation("An update of CDBurnerXP to version {0} is available. Would you like to update now?" & vbCrLf & vbCrLf & "Your current version is: {1}", updateFileVersion, Application.ProductVersion), Application.ProductName, MessageBoxButtons.YesNo, MessageBoxIcon.Information) = DialogResult.Yes Then
              Dim info As New ProcessStartInfo
              Process.Start("Updater.exe", String.Join(" ", AutoUpdate.EscapeShellArgument(args)))
              Return
            End If
          End If
        End If
      Else
        argumentList.Remove("/UpdateDone")
        args = argumentList.ToArray()
      End If
    Catch ex As Exception
    End Try

    Application.EnableVisualStyles()
    Application.SetCompatibleTextRenderingDefault(False)

    ' Verify, that fonts are working correctly
    Try
      Dim test As New System.Drawing.Font("Arial", 10)
      test = New System.Drawing.Font("Tahoma", 10)
      test = New System.Drawing.Font("MS Sans Serif", 10)
    Catch ex As ArgumentException
      If (MessageBox.Show("CDBurnerXP cannot be started, because some fonts are corrupted or not installed on your system." & Environment.NewLine & _
                      "Please check http://cdburnerxp.se/help/kb/17 for details." & Environment.NewLine & _
                      "Do you want to visit this website now?", Application.ProductName, MessageBoxButtons.YesNo, MessageBoxIcon.Error) = DialogResult.Yes) Then
        Try
          Process.Start("http://cdburnerxp.se/help/kb/17")
        Catch
        End Try
      End If

      Exit Sub
    End Try

    ' Allow users to switch to SPTD to check if this solves problems that may occur with SPTI
    If ApplicationSettings.UseSPTD Then
      BurningEngine.DeviceDriver = RocketDivision.StarBurnX.STARBURN_TRANSPORT.STARBURN_TRANSPORT_SPTD
    End If

    TaskBarExtensions.Attach()

    'Deal with command arguments
    If args.Length <> 0 Then
      Select Case args(0).ToString
        Case "/od"
          'open saved data file
          g_sAction = "OpenData"
          If args.Length > 1 AndAlso File.Exists(args(1).ToString) Then
            g_sActionFile = args(1).ToString
          End If
        Case "/oa"
          'open saved audio file
          g_sAction = "OpenAudio"
          If args.Length > 1 AndAlso File.Exists(args(1).ToString) Then
            g_sActionFile = args(1).ToString
          End If
        Case "/oi"
          'open an iso file
          g_sAction = "OpenISO"
          If args.Length > 1 AndAlso File.Exists(args(1).ToString) Then
            g_sActionFile = args(1).ToString
          End If
        Case Else
          g_sActionFile = args(0).ToString()
          Dim ext As String = Path.GetExtension(g_sActionFile)
          If ext = ".dxp" Then
            g_sAction = "OpenData"
          ElseIf ext = ".axp" Then
            g_sAction = "OpenAudio"
          ElseIf ext = ".iso" Then
            g_sAction = "OpenISO"
            g_sActionFile = args(0).ToString
          Else
            g_sAction = "OpenDataFiles"
            g_sActionFile = String.Join("?", args) + "?"
          End If
      End Select
    End If

    AddHandler Application.ThreadException, AddressOf OnUnhandledException
    AddHandler Application.ApplicationExit, AddressOf OnApplicationExit
    AddHandler AppDomain.CurrentDomain.UnhandledException, AddressOf OnUnhandledDomainException

    ' Win2000 fix
    ToolStripManager.Renderer = New CDBurnerXP.Forms.Win2kToolstripRenderer

    ' Set special delegates as "up-references"
    PathEx.TempPathGetter = AddressOf GetConfiguredTempPath

    'Set Language
    If String.IsNullOrEmpty(ApplicationSettings.AppLanguage) Then
      Dim defaultLang As String = CStr(Settings.GetValue("DefaultLanguage"))

      Using lang As New frmLanguageSelect
        Dim availableLanguages As CultureInfo() = lang.AvailableLanguages
        If availableLanguages.Length > 1 AndAlso String.IsNullOrEmpty(defaultLang) Then
          lang.ShowDialog()
          ApplicationSettings.AppLanguage = lang.Language
        ElseIf Not String.IsNullOrEmpty(defaultLang) Then
          ApplicationSettings.AppLanguage = defaultLang
        Else
          ' If only one language is installed (except en-US), use it
          If availableLanguages.Length > 0 Then
            ApplicationSettings.AppLanguage = availableLanguages(0).Name
          Else
            ApplicationSettings.AppLanguage = "en"
          End If
        End If
      End Using
    End If

    Try
      Thread.CurrentThread.CurrentUICulture = New CultureInfo(ApplicationSettings.AppLanguage)
    Catch ex As Exception
      Thread.CurrentThread.CurrentUICulture = New CultureInfo("")
    End Try

    'Start listen for media changes
    MediaNotifier.Enabled = True

    SetTranslations()
    BurnVideoDvdDialog.Animations = MainToolStrip.Animations

    Try
      BurningEngine.InitialiseStarBurn()
    Catch ex As Exception
      MessageBox.Show("The StarBurn burning engine could not be initialised. Some parts of CDBurnerXP may not work. Make sure that CDBurnerXP is installed properly.", Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Error)
    End Try

    If g_sAction = "OpenISO" Then
      Application.Run(New frmCDDVDBurnIso(Nothing, g_sActionFile))
    Else
      Application.Run(New frmSelectProject())
    End If

    Try
      BurningEngine.StarBurnEngine.Uninitialize()
    Catch
      ' Ignore errors on exit
    End Try
  End Sub

  Public Sub OnApplicationExit(ByVal sender As Object, ByVal e As EventArgs)

    Try
      ' Make sure that cache-data is cleaned
      ' on a regular basis
      PathEx.TryDeleteDirectory(PathEx.RemoveableMediaCache)
      PathEx.TryDeleteDirectory(PathEx.GetTempRipPath())
    Catch ex As Exception
      ' ignore
    End Try

    Try
      ' Try to save settings. Fails if no write permission in user folder. Rare, but shit happens.
      ApplicationSettings.Save()
    Catch ex As Exception
      MessageBox.Show(BaseForm.RM.GetString("Application settings can not be saved:") & " " & ex.Message, Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Error)
    End Try

    ' don't know if it is necessary, but it's safe
    RemoveHandler Application.ApplicationExit, AddressOf OnApplicationExit
    RemoveHandler Application.ThreadException, AddressOf OnUnhandledException

    MediaNotifier.Enabled = False
  End Sub

  Public Sub OnUnhandledDomainException(ByVal sender As Object, ByVal e As UnhandledExceptionEventArgs)
    Try
      CDBurnerXP.IO.Debug.LogError(CType(e.ExceptionObject, Exception))
    Catch ex As Exception
      ' Try to log error if possible
    End Try
  End Sub

  Public Sub OnUnhandledException(ByVal sender As Object, ByVal e As ThreadExceptionEventArgs)
    ' Show special dialog for "unfixable" MissingMethodExceptions
    If (TypeOf e.Exception Is MissingMethodException) Then
      MessageBox.Show(BaseForm.GetResourceManager().GetString("A method could not be found in one of application's components. CDBurnerXP will not be able to function properly. Please reboot your computer in order to complete possibly unfinished installations and reinstall CDBurnerXP if that does not help."), Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Error)
      Return
    End If

    Using f As New frmErrorReport(e.Exception)
      If f.ShowDialog() = DialogResult.Abort Then
        Try
          Application.Exit()
        Catch ex As InvalidOperationException
          ' .NET bug: Collection was modified
          Try
            Application.Exit()
          Catch ex2 As InvalidOperationException
          End Try
        End Try
      End If
    End Using
  End Sub

  Public Function GetConfiguredTempPath() As String
    If ApplicationSettings.UseCustomTempFolder Then
      Return ApplicationSettings.CustomTempFolder
    End If
    Return Path.GetTempPath()
  End Function

  ''' <summary>
  ''' Sets the translations for various controls (static properties).
  ''' </summary>
  Public Sub SetTranslations()
    Try
      CDBurnerXP.Forms.BaseForm.Culture = New CultureInfo(ApplicationSettings.AppLanguage)
    Catch ex As ArgumentException
      ' Ignore invalid settings
    End Try

    Dim RM As ResourceManager = New ResourceManager("CDBurnerXP_Pro.Strings", System.Reflection.Assembly.GetEntryAssembly())

    'Set all language spec in controls
    RemainingCalculator.MinutesString = RM.GetString("{0:0} minutes")
    RemainingCalculator.MinuteString = RM.GetString("< 1 minute")
    CDBurnerXP.IO.FormatFileSize.GBName = RM.GetString("GB")
    CDBurnerXP.IO.FormatFileSize.MBName = RM.GetString("MB")
    CDBurnerXP.IO.FormatFileSize.KBName = RM.GetString("KB")
    CDBurnerXP.IO.FormatFileSize.BytesName = RM.GetString("Bytes")
    CDBurnerXP.IO.FormatFileSize.UseSingleUnit = ApplicationSettings.OnlyUseSingleUnit

    TaskDialog.Translation(TaskDialog.UIText.Cancel) = RM.GetString("Cancel")
    TaskDialog.Translation(TaskDialog.UIText.Close) = RM.GetString("Close")
    TaskDialog.Translation(TaskDialog.UIText.Yes) = RM.GetString("&Yes")
    TaskDialog.Translation(TaskDialog.UIText.No) = RM.GetString("&No")
    TaskDialog.Translation(TaskDialog.UIText.OK) = RM.GetString("OK")
    TaskDialog.Translation(TaskDialog.UIText.HideDetails) = RM.GetString("Hide details")
    TaskDialog.Translation(TaskDialog.UIText.ShowDetails) = RM.GetString("Show details")
    BurnErrorReport.Translation(BurnErrorReport.UIText.Description) = RM.GetString("An error occured while burning the disc. Most likely the disc is not usable. Usually, these errors happen if the inserted media is not compatible to the drive or of poor quality.")
    BurnErrorReport.Translation(BurnErrorReport.UIText.Title) = RM.GetString("Burning error occured")
    BurnErrorReport.Translation(BurnErrorReport.UIText.SendReport) = RM.GetString("Send this error report to help making CDBurnerXP better")
    CDBurnerXP.Controls.DeviceSelect.TranslationNoDrives = RM.GetString("No Compatible Drives")
    CDBurnerXP.Controls.SpeedSelect.MaximumSpeedLabel = RM.GetString("Maximum")

    LogListView.TimeColHeader = RM.GetString("Time:")
    LogListView.EventColHeader = RM.GetString("Event:")

    FileLayoutManager.Translation(FileLayoutManager.UIText.NewFolder) = RM.GetString("New folder")
    FileLayoutManager.Translation(FileLayoutManager.UIText.BlankDirectoryOrFile) = RM.GetString("File or directory names can not be blank.")
    FileLayoutManager.Translation(FileLayoutManager.UIText.InvalidLetters) = RM.GetString("Do not use these letters: {0}")
    FileLayoutManager.Translation(FileLayoutManager.UIText.InvalidFileName) = RM.GetString("The entry contains invalid characters and cannot be renamed.")
    FileLayoutManager.Translation(FileLayoutManager.UIText.CouldNotRenameAlreadyExists) = RM.GetString("The entry could not be renamed, because an entry with the same name already exists.")
    FileLayoutManager.Translation(FileLayoutManager.UIText.File) = RM.GetString("File")
    FileLayoutManager.Translation(FileLayoutManager.UIText.Folder) = RM.GetString("Folder")
    FileLayoutManager.Translation(FileLayoutManager.UIText.Name) = RM.GetString("Name")
    FileLayoutManager.Translation(FileLayoutManager.UIText.Size) = RM.GetString("Size")
    FileLayoutManager.Translation(FileLayoutManager.UIText.Date) = RM.GetString("Date")
    FileLayoutManager.Translation(FileLayoutManager.UIText.Type) = RM.GetString("Type")
    FileLayoutManager.Translation(FileLayoutManager.UIText.Caching) = RM.GetString("Caching {0}")
    FileLayoutManager.Translation(FileLayoutManager.UIText.AddFilesHere) = RM.GetString("Drag and drop files here or use the ""Add"" button to add files to the compilation.")

    AudioLayoutManager.Translation(AudioLayoutManager.UIText.Album) = RM.GetString("Album")
    AudioLayoutManager.Translation(AudioLayoutManager.UIText.TrackNo) = RM.GetString("Track#")
    AudioLayoutManager.Translation(AudioLayoutManager.UIText.Type) = RM.GetString("Type")
    AudioLayoutManager.Translation(AudioLayoutManager.UIText.Length) = RM.GetString("Duration")
    AudioLayoutManager.Translation(AudioLayoutManager.UIText.Bitrate) = RM.GetString("Bitrate")
    AudioLayoutManager.Translation(AudioLayoutManager.UIText.Artist) = RM.GetString("Artist")
    AudioLayoutManager.Translation(AudioLayoutManager.UIText.KBits) = RM.GetString("KBit/s")
    AudioLayoutManager.Translation(AudioLayoutManager.UIText.NeverTellAgain) = RM.GetString("&Never tell again")
    AudioLayoutManager.Translation(AudioLayoutManager.UIText.SortWarning) = RM.GetString("When sorting tracks, keep in mind that this does not only affect the view, but also the order of the tracks when burned to disc.")
    AudioLayoutManager.Translation(AudioLayoutManager.UIText.Title) = RM.GetString("Title")
    AudioLayoutManager.Translation(AudioLayoutManager.UIText.TrackListHasBeenSorted) = RM.GetString("The track list has been sorted.")
    AudioLayoutManager.Translation(AudioLayoutManager.UIText.TrackNoByTag) = RM.GetString("Track # by tags")
    AudioLayoutManager.Translation(AudioLayoutManager.UIText.FullPath) = RM.GetString("Full Path")
    AudioLayoutManager.Translation(AudioLayoutManager.UIText.FileName) = RM.GetString("Filename:").Trim(":"c)

    AudioTrack.TrackInvalidException.MessageText = RM.GetString("The audio file has an invalid format.")
    AudioTrack.FileIsDrmProtectedException.MessageText = RM.GetString("The audio file is DRM protected and cannot be burned to disc.")
    CDBurnerXP.Burning.FileTooBigException.MessageText = RM.GetString("Files larger than 4GiB (4294967296 bytes) cannot be added without UDF file system.")
  End Sub

  Public Sub SetAllDefaultPathSettings()
    Dim defaultPath As String = PathEx.TryGetFolderPath(Environment.SpecialFolder.Personal)

    If ApplicationSettings.DataCachePath = "" Then
      ApplicationSettings.DataCachePath = defaultPath
    End If
    If ApplicationSettings.DataConvertTargetPath = "" Then
      ApplicationSettings.DataConvertTargetPath = defaultPath
    End If
    If ApplicationSettings.DataCopyImagePath = "" Then
      ApplicationSettings.DataCopyImagePath = defaultPath
    End If
    If ApplicationSettings.DataExplorerPath = "" Then
      ApplicationSettings.DataExplorerPath = PathEx.TryGetFolderPath(Environment.SpecialFolder.DesktopDirectory)
    End If
    If ApplicationSettings.DataFloppyImagePath = "" Then
      ApplicationSettings.DataFloppyImagePath = defaultPath
    End If
    If ApplicationSettings.DataSaveCompilationPath = "" Then
      ApplicationSettings.DataSaveCompilationPath = defaultPath
    End If
    If ApplicationSettings.DataSaveISOPath = "" Then
      ApplicationSettings.DataSaveISOPath = defaultPath
    End If
    If ApplicationSettings.AudioExplorerPath = "" Then
      ApplicationSettings.AudioExplorerPath = defaultPath
    End If
    If ApplicationSettings.DataVideoLayoutPath = "" Then
      ApplicationSettings.DataVideoLayoutPath = defaultPath
    End If

  End Sub

End Module
