' ********************************************************************************
'     Document    :  frmCDDVDBootOptions.vb
'     Description :  A form for making discs bootable
' ********************************************************************************

Imports System.Threading
Imports System.Globalization
Imports System.Resources
Imports System.IO
Imports CDBurnerXP.Controls
Imports CDBurnerXP.Forms

Public Class frmCDDVDBootOptions
  Inherits CDBurnerXP.Forms.BaseForm

  Private m_LayoutManager As FileLayoutManager

  <CLSCompliant(False)> _
  Public Sub New(ByRef layoutManager As FileLayoutManager)
    MyBase.New()

    'This call is required by the Windows Form Designer.
    InitializeComponent()

    AcceptButton = cmdOk
    CancelButton = cmdCancel
    m_LayoutManager = layoutManager
  End Sub

  Private Sub frmBoot_Load(ByVal sender As Object, _
                           ByVal e As System.EventArgs) _
          Handles MyBase.Load
    Me.Text = RM.GetString("Boot Options")
    chkBootable.Text = RM.GetString("Make Disc Bootable")
    grpBootProperties.Text = RM.GetString("Boot Image Properties:")
    lblImagePath.Text = RM.GetString("Select Path to Boot Image:")
    lblEmulationType.Text = RM.GetString("Emulation Type:")
    lblISOLevel.Text = RM.GetString("ISO Level:")
    lblLoadSegment.Text = RM.GetString("Load Segment:")
    lblSectors.Text = RM.GetString("Sectors:")
    lblPlatform.Text = RM.GetString("Platform:")
    chkISOFileDelimeters.Text = RM.GetString( _
            "Disable ISO Version Number Extension (;1)")
    chkAllowLongJolietNames.Text = RM.GetString( _
            "Enforce ISO Level1 (8+3 char max.)")
    lblGetBootHelp.Text = RM.GetString("Which options do I have to set?")
    cmdCreateFloppyImage.Text = RM.GetString("Create Floppy Image...")
    cmdOk.Text = RM.GetString("OK")
    cmdCancel.Text = RM.GetString("Cancel")

    'Load Emulation Combo Box
    LoadCombos()

    'Show Boot Information
    EnableControls(False)
    PopulateBootOptions()
  End Sub

  Private Sub LoadCombos()
    'Add the items for Emulation type
    cboEmulationType.Items.Add(New TaggedComboBoxItem(RM.GetString( _
            "Floppy 1.20MB"), NMSDVDXLib.eMediaTypeEmulationX.mteFloppy1_20MB))
    cboEmulationType.Items.Add(New TaggedComboBoxItem(RM.GetString( _
            "Floppy 1.44MB (95/98/ME boot images)"), _
            NMSDVDXLib.eMediaTypeEmulationX.mteFloppy1_44MB))
    cboEmulationType.Items.Add(New TaggedComboBoxItem(RM.GetString( _
            "Floppy 2.88MB"), NMSDVDXLib.eMediaTypeEmulationX.mteFloppy2_88MB))
    cboEmulationType.Items.Add(New TaggedComboBoxItem(RM.GetString( _
            "Hard Disc"), NMSDVDXLib.eMediaTypeEmulationX.mteHardDisc))
    cboEmulationType.Items.Add(New TaggedComboBoxItem(RM.GetString( _
            "No Emulation (NT/2000/XP boot images)"), _
            NMSDVDXLib.eMediaTypeEmulationX.mteNoEmulation))

    'Add the items for ISOLevel
    cboISOLevel.Items.Add(New TaggedComboBoxItem(RM.GetString("Level 1 (8+3)"), _
            NMSDVDXLib.eISOLevelX.isoLevel01))
    cboISOLevel.Items.Add(New TaggedComboBoxItem(RM.GetString("Level 2 (31)"), _
            NMSDVDXLib.eISOLevelX.isoLevel02))
    cboISOLevel.Items.Add(New TaggedComboBoxItem( _
            RM.GetString("ISO9660::1999 (unrestricted)"), NMSDVDXLib.eISOLevelX.iso9660_1999))

    'Add the items for Platform
    cboPlatform.Items.Add(New TaggedComboBoxItem("80x86 PC", _
            NMSDVDXLib.eBootPlatformX.btpPC80_86))
    cboPlatform.Items.Add(New TaggedComboBoxItem("MACINTOSH", _
            NMSDVDXLib.eBootPlatformX.btpMac))
    cboPlatform.Items.Add(New TaggedComboBoxItem("POWER PC", _
            NMSDVDXLib.eBootPlatformX.btpPowerPC))

  End Sub

  Private Sub PopulateBootOptions()

    Dim bootOptions As NMSDVDXLib.BootOptionsX

    'Bootable ?
    chkBootable.Checked = m_LayoutManager.DataImage.GetBootableMode()

    'Get correct ISOLevel
    'Use level1 or level 2
    Select Case m_LayoutManager.DataImage.ISOLevel

      Case NMSDVDXLib.eISOLevelX.isoLevel01
        cboISOLevel.SelectedIndex = 0

      Case NMSDVDXLib.eISOLevelX.isoLevel02 'Required for 2000/XP OS boot images
        cboISOLevel.SelectedIndex = 1

      Case NMSDVDXLib.eISOLevelX.iso9660_1999
        cboISOLevel.SelectedIndex = 2

      Case Else
        'Do nothing

    End Select

    'Delimiters?
    chkISOFileDelimeters.Checked = Not m_LayoutManager.DataImage.UseVersionDelimiter

    'Allow long Joliet names (almost never for max compatibility 207 Unicode char filenames)
    chkAllowLongJolietNames.Checked = m_LayoutManager.DataImage.UseLongJolietNames

    'Get the boot options object
    bootOptions = m_LayoutManager.DataImage.GetBootOptions()

    'Set boot image path
    txtBootImagePath.Text = bootOptions.BootFilePath

    'Show as hex
    txtLoadSegment.Text = Hex(bootOptions.LoadSegmentWord)
    txtSectors.Text = CStr(bootOptions.SectorCount)

    'Set Platform
    Select Case bootOptions.BootPlatform

      Case NMSDVDXLib.eBootPlatformX.btpMac
        cboPlatform.SelectedIndex = 1

      Case NMSDVDXLib.eBootPlatformX.btpPC80_86
        cboPlatform.SelectedIndex = 0

      Case NMSDVDXLib.eBootPlatformX.btpPowerPC
        cboPlatform.SelectedIndex = 2

      Case Else
        'Do nothing

    End Select

    'Get correct emulation mode
    Select Case bootOptions.MediaTypeEmulation

      Case NMSDVDXLib.eMediaTypeEmulationX.mteFloppy1_20MB
        cboEmulationType.SelectedIndex = 0

      Case NMSDVDXLib.eMediaTypeEmulationX.mteFloppy1_44MB 'Used for 95/98 images
        cboEmulationType.SelectedIndex = 1

      Case NMSDVDXLib.eMediaTypeEmulationX.mteFloppy2_88MB
        cboEmulationType.SelectedIndex = 2

      Case NMSDVDXLib.eMediaTypeEmulationX.mteHardDisc
        cboEmulationType.SelectedIndex = 3

      Case NMSDVDXLib.eMediaTypeEmulationX.mteNoEmulation 'Used for NT/2000/XP boot images
        cboEmulationType.SelectedIndex = 4

      Case Else
        'Do nothing
    End Select

  End Sub

  Private Function SetBootOptions() As Boolean

    Dim bEnableBoot As Boolean
    Dim oBootOptions As NMSDVDXLib.BootOptionsX

    'ISO compliance level - Only applies to
    'ISO9660 portion of image.
    'Non-ISO9660 Bridge formats (UDF/Joliet)
    'are not affected by this setting
    ' - isoLevel01 8+3 restricted names - Best for ISO9660 only
    ' - isoLevel02 31 char max name - Good for ISO9660/Joliet required for some boot images
    ' - iso9660_1999 Relaxed but not widely supported
    'oDataImage.ISOLevel = GetItemData(cboISOLevel, cboISOLevel.SelectedIndex)
    m_LayoutManager.DataImage.ISOLevel = CType(DirectCast(cboISOLevel.SelectedItem, TaggedComboBoxItem).Tag, NMSDVDXLib.eISOLevelX)
    'Delimiters? Some OS boot images (NT/2000/XP) need delimiters disabled
    m_LayoutManager.DataImage.UseVersionDelimiter = Not chkISOFileDelimeters.Checked

    'Allow long Joliet names (almost never for max compatibility 207 Unicode char filenames)
    m_LayoutManager.DataImage.UseLongJolietNames = (chkAllowLongJolietNames.CheckState = _
            CheckState.Checked)

    'Get the boot options object
    oBootOptions = m_LayoutManager.DataImage.GetBootOptions()

    'Bootable ?
    bEnableBoot = (chkBootable.CheckState = CheckState.Checked)

    'Set boot options
    oBootOptions.BootFilePath = (txtBootImagePath.Text).Trim()

    'Convert from  hex to (WORD/SHORT)
    oBootOptions.LoadSegmentWord = Convert.ToInt16(Val("&H" & (txtLoadSegment.Text).Trim()))

    'Sectors
    oBootOptions.SectorCount = Convert.ToInt16(txtSectors.Text)

    'Set Platform
    oBootOptions.BootPlatform = CType(DirectCast(cboPlatform.SelectedItem, TaggedComboBoxItem).Tag, NMSDVDXLib.eBootPlatformX)

    ' check volumne type
    If bEnableBoot AndAlso oBootOptions.BootPlatform = NMSDVDXLib.eBootPlatformX.btpPC80_86 And m_LayoutManager.DataImage.VolumeType <> NMSDVDXLib.eVolumeTypeX.vtISO9660_Joliet Then
      If MessageBox.Show(RM.GetString("For burning a Windows boot disc, you should change the volumne type to ISO9660/Joilet. Do you want to change the volumne type now?"), Application.ProductName, MessageBoxButtons.YesNo, MessageBoxIcon.Question, MessageBoxDefaultButton.Button1) = Windows.Forms.DialogResult.Yes Then
        m_LayoutManager.DataImage.VolumeType = NMSDVDXLib.eVolumeTypeX.vtISO9660_Joliet
      End If
    End If

    'Get correct emulation mode
    oBootOptions.MediaTypeEmulation = CType(DirectCast(cboEmulationType.SelectedItem, TaggedComboBoxItem).Tag, NMSDVDXLib.eMediaTypeEmulationX)

    'Enable the bootable mode
    'The image will verify existance of the boot image file
    If m_LayoutManager.DataImage.SetBootableMode(bEnableBoot) = False Then
      MessageBox.Show(Me, RM.GetString( _
              "Boot options could not be set, please check the options!"), Application.ProductName, _
              MessageBoxButtons.OK, MessageBoxIcon.Error)
      m_LayoutManager.UpdateRootIcon()
      Return False

    End If

    m_LayoutManager.SetCompilationChanged(True)
    m_LayoutManager.UpdateRootIcon()

    Return True
  End Function

  Private Sub EnableControls(ByVal blnEnable As Boolean)
    grpBootProperties.Enabled = blnEnable
  End Sub

  Private Sub cmdOk_Click(ByVal sender As System.Object, _
                          ByVal e As System.EventArgs) _
          Handles cmdOk.Click

    If SetBootOptions() Then
      Me.Close()
    End If

  End Sub

  Private Sub cmdCancel_Click(ByVal sender As System.Object, _
                              ByVal e As System.EventArgs) _
          Handles cmdCancel.Click
    Me.Close()
  End Sub

  Private Sub cmdSelectImage_Click(ByVal sender As System.Object, _
                                   ByVal e As System.EventArgs) _
          Handles cmdSelectImage.Click
    With OpenImageDialog
      .Title = RM.GetString("Select boot image file")
      .Filter = RM.GetString("Boot Image File") & " (*.img, *.ima)|*.img;*.ima;|" _
              + RM.GetString("All files") & "|*.*"
      .Multiselect = False

      If SafeOpenFileDialog.ShowDialog(OpenImageDialog, Me, "OpenBootImage") = Windows.Forms.DialogResult.Cancel Then
        Exit Sub

      Else
        txtBootImagePath.Text = .FileName
      End If

    End With
  End Sub

  Private Sub chkBootable_CheckedChanged(ByVal sender As System.Object, _
                                         ByVal e As System.EventArgs) _
          Handles chkBootable.CheckedChanged

    If chkBootable.Checked Then
      EnableControls(True)

    Else
      EnableControls(False)
    End If

  End Sub

  Private Sub cmdCreateFloppyImage_Click(ByVal sender As System.Object, _
                                         ByVal e As System.EventArgs) _
          Handles cmdCreateFloppyImage.Click

    Using frmFloppyImage As New frmCDDVDImageFromFloppy
      frmFloppyImage.ShowDialog(Me)

      If frmFloppyImage.ImagePath <> "" Then

        If MessageBox.Show(Me, RM.GetString( _
                "Do you want to use the image you just created as boot image?"), _
                Application.ProductName, MessageBoxButtons.YesNo, MessageBoxIcon.Question, _
                MessageBoxDefaultButton.Button1) = Windows.Forms.DialogResult.Yes Then
          txtBootImagePath.Text = frmFloppyImage.ImagePath
        End If

      End If
    End Using
  End Sub

End Class
