'*************************************************************************
' Klass: frmCDDVDConvertISO
' Beroenden: 
' Syfte: 
' F�rfattare: Fredrik Haglund, Stefan Haglund
' Senast �ndrad av: Stefan Haglund
' Senast �ndrad: 2007-04-24 18:12
' Kommentarer: 
'*************************************************************************

Imports System.Threading
Imports System.Globalization
Imports System.Resources
Imports System.IO
Imports CDBurnerXP.IO
Imports CDBurnerXP.Forms
Imports CDBurnerXP

Public Class frmCDDVDConvertISO
  Private WithEvents ConvertIso As New ISOTools
  Private isConverting As Boolean = False

  Public Sub New()
    ' This call is required by the Windows Form Designer.
    InitializeComponent()
    AcceptButton = cmdConvertToISO
    CancelButton = cmdCancel
    ' Add any initialization after the InitializeComponent() call.

  End Sub

  Private Sub cmdSelectImagePath_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdSelectImagePath.Click
    With OpenImageDialog
      .Title = RM.GetString("Select image to convert")
      .Filter = RM.GetString("Image files") & " (*.bin;*.nrg)|*.bin;*.nrg"
      If SafeOpenFileDialog.ShowDialog(OpenImageDialog, Me, "OpenIsoToConvert") = System.Windows.Forms.DialogResult.Cancel Then Exit Sub
      txtImageFile.Text = .FileName
      cmdSelectISOFolder.Enabled = True
    End With
  End Sub

  Private Sub cmdSelectISOFolder_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdSelectISOFolder.Click
    With SaveImageDialog
      .Filter = RM.GetString("Image files") & " (*.iso)|*.iso"
      .FileName = Path.ChangeExtension(Path.GetFileName(txtImageFile.Text), ".iso")
      .InitialDirectory = ApplicationSettings.DataConvertTargetPath
      If .ShowDialog(Me) = System.Windows.Forms.DialogResult.Cancel Then Exit Sub
      ApplicationSettings.DataConvertTargetPath = Path.GetDirectoryName(.FileName)
      txtISOPath.Text = .FileName
    End With
  End Sub

  Private Sub cmdConvertToISO_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdConvertToISO.Click
    If txtImageFile.Text = "" Then
      MessageBox.Show(Me, RM.GetString("You did not select an image file for conversion." & vbCrLf & vbCrLf & "Select a file that is to be converted to an ISO image in order to continue."), Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.None, MessageBoxDefaultButton.Button1)
      Exit Sub
    End If
    ConvertIso.InputFilename = txtImageFile.Text

    Try
      If IsNrgAudioImage(txtImageFile.Text) Then
        MessageBox.Show(Me, String.Format(RM.GetString("The selected NRG file contains an audio disc image and cannot be converted to an ISO image."), Path.GetFileName(txtImageFile.Text)), Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Error, MessageBoxDefaultButton.Button1)
        Exit Sub
      End If
    Catch ex As FileNotFoundException
      Dim msg As String = String.Format(RM.GetString("The file '{0}' does not exist."), txtImageFile.Text)
      MessageBox.Show(Me, msg, Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Error)
      Exit Sub
    Catch ex As IOException
      Dim msg As String = String.Format(RM.GetString("Cannot read the file '{0}'. It may be in use by another application."), txtImageFile.Text)
      MessageBox.Show(Me, msg, Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Error)
      Exit Sub
    End Try

    If txtISOPath.Text = "" Then
      MessageBox.Show(Me, RM.GetString("No target folder for the converted ISO file has been selected."), Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.None, MessageBoxDefaultButton.Button1)
      Exit Sub
    End If

    ConvertIso.OutputFilename = txtISOPath.Text
    ' Check for enough free disc space before
    If Not DiskSpaceRequiredDialog.Show(Me, PathEx.TryGetFileSize(txtImageFile.Text), txtISOPath.Text) Then
      Exit Sub
    End If

    cmdConvertToISO.Enabled = False
    isConverting = True
    If ConvertIso.ConvertImages() Then
      Application.DoEvents()
      MessageBox.Show(Me, String.Format(RM.GetString("ISO file {0} was created sucessfully."), ConvertIso.OutputFilename), Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Information, MessageBoxDefaultButton.Button1)
      Application.DoEvents()
    Else
      If ConvertIso.CancelConvert Then
        ConvertIso.CancelConvert = False
        lblStatus.Text = RM.GetString("Status: Idle")
        prgISOConversion.Value = 0
        Application.DoEvents()
        MessageBox.Show(Me, RM.GetString("ISO file creation was cancelled."), Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Error, MessageBoxDefaultButton.Button1)
        Application.DoEvents()
      End If
    End If
    isConverting = False
    cmdConvertToISO.Enabled = True

    prgISOConversion.Value = 0
  End Sub

  Private Sub ConvertIso_ConvertProgress(ByVal Percent As Double) Handles ConvertIso.ConvertProgress
    If Convert.ToInt32(Percent) > prgISOConversion.Value Then
      prgISOConversion.Value = Convert.ToInt32(Percent)
      lblStatus.Text = String.Format(RM.GetString("Status: Converting to ISO {0}% Done"), Convert.ToInt32(Percent).ToString)
      lblStatus.Refresh()
      prgISOConversion.Refresh()
    End If
  End Sub

  Private Sub frmCDDVDConvertISO_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles MyBase.Load
    Me.Text = RM.GetString("Convert Image To ISO")
    lblImagePath.Text = RM.GetString("Path to image to be converted:")
    lblISODirectory.Text = RM.GetString("Path to ISO image:")
    lblStatus.Text = RM.GetString("Status: Idle")
    cmdConvertToISO.Text = RM.GetString("Convert To ISO")
    cmdCancel.Text = RM.GetString("Cancel")
    cmdSelectISOFolder.Enabled = False
  End Sub

  Private Sub cmdCancel_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdCancel.Click
    If isConverting Then
      ConvertIso.CancelConvert = True
    Else
      Me.Close()
    End If
  End Sub

  Private Sub frmCDDVDConvertISO_Closing(ByVal sender As Object, ByVal e As System.ComponentModel.CancelEventArgs) Handles MyBase.Closing
    If isConverting Then
      e.Cancel = True
    End If
  End Sub
End Class
