'*************************************************************************
' Klass: frmCDDVDConvertISO
' Beroenden: 
' Syfte: 
' F�rfattare: Fredrik Haglund, Stefan Haglund
' Senast �ndrad av: Stefan Haglund
' Senast �ndrad: 2007-04-24 18:16
' Kommentarer: 
'*************************************************************************

Imports System.IO
Imports System.Threading
Imports System.Globalization
Imports System.Resources
Imports CDBurnerXP.Forms
Imports CDBurnerXP

Public Class frmCDDVDImageFromFloppy
  Private WithEvents ImageTool As New ISOTools
  Private m_bExtractCancelled As Boolean = False
  Private m_bUseImageCreated As Boolean = False
  Private isExtracting As Boolean = False
  Private _ImagePath As String = ""

  Public Sub New()
    MyBase.New()

    'This call is required by the Windows Form Designer.
    InitializeComponent()
    AcceptButton = cmdCreateImage
    CancelButton = cmdCancel
    'Add any initialization after the InitializeComponent() call
  End Sub

  Public ReadOnly Property ImagePath() As String
    Get
      If m_bUseImageCreated Then
        Return _ImagePath
      Else
        Return ""
      End If
    End Get
  End Property

  Private Sub frmCDDVDFloppyImage_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles MyBase.Load
    'Language Set
    Me.Text = RM.GetString("Create Floppy Image")
    lblSelectDrive.Text = RM.GetString("Select Removable Drive:")
    lblImagePath.Text = RM.GetString("Select Path To The Created Image:")
    lblStatus.Text = RM.GetString("Status: Idle")
    cmdCreateImage.Text = RM.GetString("Create Image")
    cmdCancel.Text = RM.GetString("Cancel")

    Dim strDrives As String() = Environment.GetLogicalDrives()
    For Each strDrive As String In strDrives
      If ImageTool.isFloppy(strDrive) Then
        cboFloppy.Items.Add(New TaggedComboBoxItem(strDrive, strDrive))
      End If
    Next
    If cboFloppy.Items.Count = 0 Then
      cboFloppy.Items.Add(RM.GetString("No Floppy Drives Found"))
      cboFloppy.SelectedIndex = 0
      cmdCreateImage.Enabled = False
      cmdSelectPath.Enabled = False
    Else
      cboFloppy.SelectedIndex = 0
    End If
  End Sub

  Private Sub cmdCreateImage_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdCreateImage.Click
    m_bExtractCancelled = False
    If Not Directory.Exists(CType(cboFloppy.SelectedItem, TaggedComboBoxItem).Name) Then
      MessageBox.Show(Me, String.Format(RM.GetString("Please insert a disc into drive {0}"), CType(cboFloppy.SelectedItem, TaggedComboBoxItem).Name), "CDBurnerXP 3", MessageBoxButtons.OK, MessageBoxIcon.Error, MessageBoxDefaultButton.Button1)
      Exit Sub
    End If
    If txtImagePath.Text = "" Then
      MessageBox.Show(Me, RM.GetString("No Image File Selected."), Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Error, MessageBoxDefaultButton.Button1)
      Exit Sub
    End If
    isExtracting = True
    cmdCreateImage.Enabled = False
    cmdCancel.Enabled = True
    cboFloppy.Enabled = False
    cmdSelectPath.Enabled = False
    lblStatus.Text = String.Format(RM.GetString("Status: Extracting Image from {0}"), CType(cboFloppy.SelectedItem, TaggedComboBoxItem).Name)
    Try
      ImageTool.ReadImageFromFloppyDevice(CType(cboFloppy.SelectedItem, TaggedComboBoxItem).Name, txtImagePath.Text)
    Catch ex As Exception
      MessageBox.Show(Me, ex.Message, Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Error, MessageBoxDefaultButton.Button1)
    End Try
    cmdCreateImage.Enabled = True
    lblStatus.Text = RM.GetString("Status: Idle")
    cmdCancel.Enabled = True
    cboFloppy.Enabled = True
    cmdSelectPath.Enabled = True
    isExtracting = False
  End Sub

  Private Sub cmdCancel_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdCancel.Click
    If isExtracting Then
      ImageTool.CancelExtract = True
      Exit Sub
    Else
      If Not m_bExtractCancelled Then
        _ImagePath = txtImagePath.Text
      Else
        _ImagePath = ""
      End If
      Me.Close()
    End If
  End Sub

  Private Sub ImageTool_ExtractImageProgress(ByVal Percent As Single) Handles ImageTool.ExtractImageProgress
    prgProgress.Value = Convert.ToInt32(Percent)
  End Sub

  Private Sub cmdSelectPath_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdSelectPath.Click
    With SaveImageDialog
      .InitialDirectory = ApplicationSettings.DataFloppyImagePath
      .Title = RM.GetString("Save Image As...")
      .AddExtension = True
      .Filter = RM.GetString("Image files") & " (*.img)|*.img"
      .DefaultExt = ".img"
      If .ShowDialog(Me) = System.Windows.Forms.DialogResult.Cancel Then Exit Sub
      txtImagePath.Text = .FileName
      ApplicationSettings.DataFloppyImagePath = Path.GetDirectoryName(.FileName)
    End With
  End Sub

  Private Sub ImageTool_ExtractCancelled() Handles ImageTool.ExtractCancelled
    MessageBox.Show(Me, RM.GetString("Image Extraction Cancelled"), Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Information, MessageBoxDefaultButton.Button1)
    prgProgress.Value = 0
    cmdCreateImage.Enabled = True
    ImageTool.CancelExtract = False
    If File.Exists(txtImagePath.Text) Then
      File.Delete(txtImagePath.Text)
    End If
    m_bExtractCancelled = True
  End Sub

  Protected Overrides Sub Finalize()
    MyBase.Finalize()
  End Sub

  Private Sub frmCDDVDFloppyImage_Closing(ByVal sender As Object, ByVal e As System.ComponentModel.CancelEventArgs) Handles MyBase.Closing
    If isExtracting Then
      e.Cancel = True
      Exit Sub
    End If
  End Sub

  Private Sub ImageTool_ExtractDone() Handles ImageTool.ExtractDone
    MessageBox.Show(Me, RM.GetString("Extracting floppy image was successfully completed."), Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Information, MessageBoxDefaultButton.Button1)
    m_bUseImageCreated = True
    Application.DoEvents()
  End Sub
End Class
