'*************************************************************************
' Klass: frmPrintCover
' Beroenden: 
' Syfte: 
' F�rfattare: Fredrik Haglund, Stefan Haglund
' Senast �ndrad av: Stefan Haglund
' Senast �ndrad: 2007-04-26 08:33
' Kommentarer: 
'*************************************************************************

Imports System.Drawing.Printing
Imports System.Math
Imports System.Threading
Imports System.Globalization
Imports System.Resources
Imports System.IO
Imports CDBurnerXP.Printing
Imports HewlettPackard
Imports CDBurnerXP.Forms
Imports CDBurnerXP

Public Class frmPrintCover
  Private WithEvents m_Cover As New CoverPrintDocument
  Private prd As New PrintDialog
  Private ppd As New PrintPreviewDialog
  Private psd As New PageSetupDialog
  Private m_AllowAudioDisplayFormats As Boolean = False
  Private dataDisplayFormats As String() = {"{default}", "{filenameWithoutExtension}", "{fullPath}", "{extension}", "{filename}"}
  Private audioDisplayFormats As String() = {"{album}", "{artist}", "{title}", "{trackNo}", "{genre}"}

  Public Property AllowAudioDisplayFormats() As Boolean
    Get
      Return m_AllowAudioDisplayFormats
    End Get
    Set(ByVal value As Boolean)
      m_AllowAudioDisplayFormats = value
    End Set
  End Property

  Protected Overrides ReadOnly Property AutoTranslate() As Boolean
    Get
      Return True
    End Get
  End Property

  Public ReadOnly Property CoverPrintDocument() As CoverPrintDocument
    Get
      Return m_Cover
    End Get
  End Property

  Public Sub New(ByVal discname As String)
    ' This call is required by the Windows Form Designer.
    InitializeComponent()
    ' thank you Microsoft for your excellent products
    AcceptButton = cmdPrint
    CancelButton = cmdClose

    ' Add any initialization after the InitializeComponent() call.
    m_Cover.DocumentName = "CDCover"
    psd.Document = m_Cover
    ppd.Document = m_Cover
    prd.Document = m_Cover

    txtTitle.Text = discname
  End Sub

  Protected Overrides Sub OnLoad(ByVal e As System.EventArgs)
    MyBase.OnLoad(e)

    'Language Set
    ppd.Icon = Icon.FromHandle(My.Resources.Printer.GetHicon())
    ppd.RightToLeft = Me.RightToLeft
    ppd.RightToLeftLayout = Me.RightToLeftLayout
    sepPrintType.Text = RM.GetString("Select cover type:")
    optSlimCase.Text = RM.GetString("Slim CD case")
    optDVDCase.Text = RM.GetString("DVD case")
    optMiniCase.Text = RM.GetString("Mini CD case")
    sepCoverSettings.Text = RM.GetString("Edit cover information:")
    lblTitle.Text = RM.GetString("Title:")
    lblDate.Text = RM.GetString("Date:")
    lblNumber.Text = RM.GetString("Number:")
    lblDisplayFormat.Text = RM.GetString("&Display format:")
    cmdPreview.Text = RM.GetString("Preview")
    cmdPrint.Text = RM.GetString("Print...")
    bLightScribe.Text = RM.GetString("LightScribe...")
    optStandardBack.Text = RM.GetString("&Back")
    Me.Text = RM.GetString("Print Cover")

    ' Adjust UI to increated language string length
    If (txtTitle.Left - lblTitle.Right < 3) Then
      txtTitle.Left = lblTitle.Right + 3
    End If
    If (dteDate.Left - lblDate.Right < 3) Then
      dteDate.Left = lblDate.Right + 3
    End If
    txtTitle.Left = Math.Max(dteDate.Left, txtTitle.Left)
    dteDate.Left = Math.Max(dteDate.Left, txtTitle.Left)
    nudNo.Left = txtTitle.Right - nudNo.Width
    lblNumber.Left = nudNo.Left - 3 - lblNumber.Width
    cboDisplayFormats.Left = lblDisplayFormat.Right + 3
    cboDisplayFormats.Width = nudNo.Right - cboDisplayFormats.Left
    txtImagePath.Bounds = New Rectangle(lblSelectFrontImage.Right + 3, txtImagePath.Top, cmdProjectPath.Left - lblSelectFrontImage.Right - 6, txtImagePath.Height)
    optStandardBack.Left = optStandardCase.Right + 3

    Dim txtUsePathStart As Integer = Math.Max(lblFront.Right + 3, lblInside.Right + 3)
    txtUseCoverImage.Bounds = New Rectangle(txtUsePathStart, txtUseCoverImage.Top, bBrowseConverImageFront.Left - txtUsePathStart - 6, txtUseCoverImage.Height)
    txtUseCoverImageInside.Bounds = New Rectangle(txtUsePathStart, txtUseCoverImageInside.Top, bBrowseConverImageFront.Left - txtUsePathStart - 6, txtUseCoverImageInside.Height)

    Dim translatedDataDisplayFormats As String() = {RM.GetString("{default}"), RM.GetString("{filenameWithoutExtension}"), RM.GetString("{fullPath}"), RM.GetString("{extension}"), RM.GetString("{filename}")}
    Dim translatedAudioDisplayFormats As String() = {RM.GetString("{album}"), RM.GetString("{artist}"), RM.GetString("{title}"), RM.GetString("{trackNo}"), RM.GetString("{genre}")}

    cboDisplayFormats.Items.AddRange(translatedDataDisplayFormats)
    If m_AllowAudioDisplayFormats Then
      cboDisplayFormats.Items.AddRange(translatedAudioDisplayFormats)
      cboDisplayFormats.Items.Add(RM.GetString("{album}") & " - " & RM.GetString("{artist}") & " - " & RM.GetString("{title}"))
      cboDisplayFormats.Items.Add(RM.GetString("{artist}") & " - " & RM.GetString("{title}"))
      cboDisplayFormats.Items.Add(RM.GetString("{album}") & " - " & RM.GetString("{title}"))
      cboDisplayFormats.Items.Add(RM.GetString("{trackNo}") & ": " & RM.GetString("{title}"))
      m_Cover.UseAudioTags = True
    End If

    If LightScribe.IsSupported Then
      bLightScribe.Visible = True
    End If

    m_Cover.Translation(CoverPrintDocument.Translations.Title) = RM.GetString("TITLE: ")
    m_Cover.Translation(CoverPrintDocument.Translations.No) = RM.GetString("NO: ")
    m_Cover.Translation(CoverPrintDocument.Translations.Date) = RM.GetString("DATE: ")

    txtTitle.MaxLength = 47

    cboUseCoverImage.Checked = Conversion.ToBoolean(Settings.GetValue(Me, "UseCoverImages", False))
    cboCreateOwn.Checked = Not cboUseCoverImage.Checked
  End Sub

  Protected Overrides Sub OnClosing(ByVal e As System.ComponentModel.CancelEventArgs)
    MyBase.OnClosing(e)

    Settings.SetValue(Me, "UseCoverImages", cboUseCoverImage.Checked)
  End Sub

  Private Function GetUntranslatedDisplayFormat() As String
    Dim format As String = cboDisplayFormats.Text

    For Each varname As String In dataDisplayFormats
      format = format.Replace(RM.GetString(varname), varname)
    Next

    For Each varname As String In audioDisplayFormats
      format = format.Replace(RM.GetString(varname), varname)
    Next

    Return format
  End Function

  Private Sub cmdPreview_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdPreview.Click
    'Show the print preview dialog
    With ppd
      m_Cover.DisplayFormat = GetUntranslatedDisplayFormat()
      Try
        .WindowState = FormWindowState.Maximized
        .ShowDialog(Me)
      Catch exp As Exception
        MessageBox.Show(Me, RM.GetString("An error occurred while trying to load the document for Print Preview. Make sure you currently have access to a printer. A printer must be connected and accessible for Print Preview to work."), Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Error)
      End Try
    End With
  End Sub

  Private Sub cmdPrint_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdPrint.Click
    'Show the printer settings dialog
    With prd
      .AllowSelection = False
      .AllowSomePages = False
      m_Cover.DisplayFormat = GetUntranslatedDisplayFormat
      If .ShowDialog(Me) = System.Windows.Forms.DialogResult.OK Then
        Try
          m_Cover.Print()
        Catch er As Exception
          MessageBox.Show(Me, er.Message, Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Error)
        End Try
      End If
    End With
  End Sub

  Private Sub optStandardCase_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles optStandardCase.CheckedChanged
    m_Cover.CoverType = CoverType.Standard
    txtUseCoverImageInside.Enabled = True
    bBrowseConverImageInside.Enabled = True
    lblInside.Enabled = True
    txtTitle.MaxLength = 47
  End Sub

  Private Sub optStandardBack_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles optStandardBack.CheckedChanged
    m_Cover.CoverType = CoverType.StandardBack
    txtUseCoverImageInside.Enabled = False
    bBrowseConverImageInside.Enabled = False
    lblInside.Enabled = False
    txtTitle.MaxLength = 47
  End Sub

  Private Sub optSlimCase_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles optSlimCase.CheckedChanged
    m_Cover.CoverType = CoverType.SlimCD
    txtUseCoverImageInside.Enabled = True
    bBrowseConverImageInside.Enabled = True
    lblInside.Enabled = True
    txtTitle.MaxLength = 47
  End Sub

  Private Sub optDVDCase_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles optDVDCase.CheckedChanged
    m_Cover.CoverType = CoverType.DVD
    txtUseCoverImageInside.Enabled = False
    bBrowseConverImageInside.Enabled = False
    lblInside.Enabled = False
    txtTitle.MaxLength = 43
  End Sub

  Private Sub optMiniCase_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles optMiniCase.CheckedChanged
    m_Cover.CoverType = CoverType.MiniDisc
    txtUseCoverImageInside.Enabled = True
    bBrowseConverImageInside.Enabled = True
    lblInside.Enabled = True
    txtTitle.MaxLength = 42
  End Sub

  Private Sub cmdProjectPath_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdProjectPath.Click
    Dim path As String = BrowseForImage()
    If Not String.IsNullOrEmpty(path) Then
      txtImagePath.Text = path
      m_Cover.ImagePath = path
      cboCreateOwn.Checked = True
    End If
  End Sub

  Private Sub txtTitle_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles txtTitle.TextChanged
    m_Cover.Title = txtTitle.Text
    cboCreateOwn.Checked = True
  End Sub

  Private Sub dteDate_ValueChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles dteDate.ValueChanged
    m_Cover.Date = dteDate.Value
    cboCreateOwn.Checked = True
  End Sub

  Private Sub nudNo_ValueChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles nudNo.ValueChanged
    m_Cover.Number = Convert.ToInt32(nudNo.Value)
    cboCreateOwn.Checked = True
  End Sub

  Private Sub bLightScribe_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles bLightScribe.Click
    Dim imgPath As String = Nothing

    Try
      imgPath = LightScribe.GenerateDiscImage(txtTitle.Text, txtImagePath.Text, Convert.ToInt32(nudNo.Value))
    Catch ex As Exception
      MessageBox.Show(RM.GetString("Could not generate image for printing:") + " " + ex.Message, Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Error)
      Return
    End Try

    LightScribe.ShowPrintOptions(imgPath, True)
  End Sub

  Private Sub bBrowseConverImage_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles bBrowseConverImageFront.Click
    Dim path As String = BrowseForImage()
    If Not String.IsNullOrEmpty(path) Then
      txtUseCoverImage.Text = path
      m_Cover.ImagePath = path
      cboUseCoverImage.Checked = True
    End If
  End Sub

  Private Sub bBrowseConverImageInside_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles bBrowseConverImageInside.Click
    Dim path As String = BrowseForImage()
    If Not String.IsNullOrEmpty(path) Then
      txtUseCoverImageInside.Text = path
      m_Cover.ImageInsidePath = path
      cboUseCoverImage.Checked = True
    End If
  End Sub

  Private Function BrowseForImage() As String
    With dlgOpen
      .Filter = RM.GetString("Image files") & " (*.bmp;*.jpg;*.jpeg;*.jfif;*.gif;*.png;*.tif;*.tiff)|*.bmp;*.jpg;*.jpeg;*.jfif;*.gif;*.png;*.tif;*.tiff|" + _
                "Windows Bitmap (*.bmp)|*.bmp|" + _
                "Garphics Interchange Format (*.gif)|*.gif|" + _
                "JPEG File (*.jpg)|*.jpg;*.jpeg;*.jfif|" + _
                "Portable Networks Graphics (*.png)|*.png|" + _
                "Tag Image File (*.tif)|*.tif;*.tiff"
      .CheckFileExists = True
      .CheckPathExists = True
      .Multiselect = False
      .InitialDirectory = System.Windows.Forms.Application.StartupPath() & "\Resources\"
      If SafeOpenFileDialog.ShowDialog(dlgOpen, Me, "OpenConverImage") = System.Windows.Forms.DialogResult.OK Then
        Try
          Image.FromFile(.FileName)
          Return .FileName
        Catch ex As Exception
          MessageBox.Show(String.Format(RM.GetString("File {0} is not a valid image file."), .FileName), Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Error)
        End Try
      End If
    End With

    Return String.Empty
  End Function

  Private Sub cboUseCoverImage_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cboUseCoverImage.CheckedChanged
    m_Cover.UseCoverImage = True
  End Sub

  Private Sub cboCreateOwn_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cboCreateOwn.CheckedChanged
    m_Cover.UseCoverImage = False
  End Sub

  Private Sub cboDisplayFormats_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cboDisplayFormats.SelectedIndexChanged
    cboCreateOwn.Checked = True
  End Sub
End Class
