'*************************************************************************
' Klass: frmReportDrive
' Beroenden: 
' Syfte: 
' F�rfattare: Fredrik Haglund, Stefan Haglund
' Senast �ndrad av: Stefan Haglund
' Senast �ndrad: 2007-04-26 08:51
' Kommentarer: 
'*************************************************************************

Imports System.Net
Imports System.IO
Imports System.Threading
Imports System.Globalization
Imports System.Resources
Imports CDBurnerXP.Burning
Imports CDBurnerXP

Public Class frmReportDrive
  Private tmpDevice As NMSDVDXLib.DeviceX
  Private m_SentSuccessful As Boolean = False
  Private m_Model As String
  Private m_Vendor As String
  Private m_Bus As String
  Private m_Exception As Exception

  Public Sub New()
    ' This call is required by the Windows Form Designer.
    InitializeComponent()
    CancelButton = cmdClose
    AcceptButton = cmdReport
    ' Add any initialization after the InitializeComponent() call.
  End Sub

  Protected Overrides Sub OnLoad(ByVal e As System.EventArgs)
    MyBase.OnLoad(e)

    'Language
    Me.Text = RM.GetString("Report working drive")
    lblReport.Text = RM.GetString("By reporting your drive as working with CDBurnerXP you will help other users to find if it will work for them.")
    lblDrive.Text = RM.GetString("Select drive to report:")
    lblBusType.Text = RM.GetString("Select bus type:")
    cmdReport.Text = RM.GetString("Report Drive")
    cmdClose.Text = RM.GetString("Cancel")
    lnkPrivacy.Text = RM.GetString("Privacy Policy")
    cboBus.Items.Clear()
    cboBus.Items.AddRange(New String() {RM.GetString("Internal (IDE/EIDE/SATA)"), "Firewire", "USB", "SCSI"})

    cmdReport.Enabled = (cbDevice.Items.Count > 0)
    cboBus.SelectedIndex = 0
    cboDrive_SelectedIndexChanged(Me, Nothing)
  End Sub

  Private Sub cmdClose_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdClose.Click
    RemoveHandler bgwSendDrive.RunWorkerCompleted, AddressOf bgwSendDrive_RunWorkerCompleted
    Me.DialogResult = DialogResult.Abort
    Me.Close()
  End Sub

  Private Sub cmdReport_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdReport.Click
    m_SentSuccessful = False
    cmdReport.Enabled = False
    If Not bgwSendDrive.IsBusy Then
      bgwSendDrive.RunWorkerAsync()
    End If
  End Sub

  Private Sub bgwSendDrive_RunWorkerCompleted(ByVal sender As System.Object, ByVal e As System.ComponentModel.RunWorkerCompletedEventArgs) Handles bgwSendDrive.RunWorkerCompleted
    If m_SentSuccessful Then
      MessageBox.Show(RM.GetString("Drive report successfully sent. Thank you for sending the report to us!"), Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Information, MessageBoxDefaultButton.Button1)
      Me.Close()
    Else
      IO.Debug.LogError(m_Exception)
      MessageBox.Show(RM.GetString("Error while trying to send drive report.") & " " & m_Exception.Message, Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Error, MessageBoxDefaultButton.Button1)
    End If
    cmdReport.Enabled = True
  End Sub

  Private Sub bgwSendDrive_DoWork(ByVal sender As System.Object, ByVal e As System.ComponentModel.DoWorkEventArgs) Handles bgwSendDrive.DoWork
    Dim postData As String
    Dim request As HttpWebRequest
    Dim response As HttpWebResponse

    postData = "model=" & Utility.UrlEncode(m_Model) & "&vendor=" & Utility.UrlEncode(m_Vendor) & "&type=" & Utility.UrlEncode(m_Bus) & "&version=" & Utility.UrlEncode(Application.ProductVersion)
    Dim postBytes As Byte() = System.Text.Encoding.ASCII.GetBytes(postData)

    request = CType(WebRequest.Create("http://cdburnerxp.se/submit_drive.php"), HttpWebRequest)
    request.ContentType = "application/x-www-form-urlencoded"
    request.ContentLength = postBytes.Length
    request.Method = "POST"
    request.AllowAutoRedirect = True
    request.Timeout = 5000
    Try
      Dim requestStream As Stream = request.GetRequestStream()
      requestStream.Write(postBytes, 0, postBytes.Length)
      requestStream.Close()
      response = CType(request.GetResponse(), HttpWebResponse)
      m_SentSuccessful = True
      Debug.WriteLine(response.Headers)
    Catch ex As Exception
      m_Exception = ex
      Exit Sub
    End Try
  End Sub

  Private Sub cboDrive_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cbDevice.SelectedIndexChanged
    If cbDevice.SelectedDevice IsNot Nothing Then
      m_Model = cbDevice.SelectedDevice.GetModel()
      m_Vendor = cbDevice.SelectedDevice.GetVendor()
      cmdReport.Enabled = True
    Else
      cmdReport.Enabled = False
    End If
  End Sub

  Private Sub cboBus_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cboBus.SelectedIndexChanged
    m_Bus = CStr(IIf(cboBus.SelectedIndex = 0, "internal", cboBus.SelectedItem))
  End Sub
End Class
