'*************************************************************************
' Klass: frmSelectProject
' Beroenden: 
' Syfte: 
' F�rfattare: Fredrik Haglund, Stefan Haglund
' Senast �ndrad av: Stefan Haglund
' Senast �ndrad: 2007-04-26 08:53
' Kommentarer: 
'*************************************************************************
Imports System.Threading
Imports System.Globalization
Imports System.Resources
Imports System.IO
Imports CDBurnerXP.Forms
Imports CDBurnerXP.Controls
Imports CDBurnerXP.Burning
Imports CDBurnerXP.Net
Imports Microsoft.Win32
Imports CDBurnerXP
Imports Microsoft.Win32.Windows7

Public Class frmSelectProject
  Private m_IgnoreNotify As Boolean
  Private Shared m_Instance As frmSelectProject

  Public Sub New()
    ' This call is required by the Windows Form Designer.
    m_Instance = Me
    InitializeComponent()

    AcceptButton = bOK
    CancelButton = cmdExit
    ' Add any initialization after the InitializeComponent() call.

    cmnuRestore.Text = RM.GetString("Open CDBurnerXP")
    cmnuClose.Text = RM.GetString("Close CDBurnerXP")
    ntyMain.ContextMenu = Me.cmnuNotify
  End Sub

  Protected Overrides ReadOnly Property AutoTranslate() As Boolean
    Get
      Return True
    End Get
  End Property

  Public Shared ReadOnly Property Instance() As frmSelectProject
    Get
      Return m_Instance
    End Get
  End Property

  Protected Overrides Sub OnLoad(ByVal e As System.EventArgs)
    MyBase.OnLoad(e)

    ' For whatever reason, it is not possible to set this propery automatically
    ' in the list box item in the Text property.
    pnlDataDisc.AccessibleName = pnlDataDisc.Text.Replace("&", "")
    pnlAudioDisc.AccessibleName = pnlAudioDisc.Text.Replace("&", "")
    pnlBurnIso.AccessibleName = pnlBurnIso.Text.Replace("&", "")
    pnlCopyDisc.AccessibleName = pnlCopyDisc.Text.Replace("&", "")
    pnlEraseDisc.AccessibleName = pnlEraseDisc.Text.Replace("&", "")
    pnlVideoDisc.AccessibleName = pnlVideoDisc.Text.Replace("&", "")
  End Sub

  Protected Overrides Sub OnLoaded(ByVal sender As Object, ByVal e As System.EventArgs)
    lbSelectProject.Focus()

    If ApplicationSettings.AutoUpdate Then
      AutoUpdate.BeginCheckAndDownloadUpdate()
    End If

    'Do we have any command arguments?
    If g_sAction <> "" Then
      Select Case g_sAction
        Case "OpenData"
          StartData()
        Case "OpenDataFiles"
          StartData()
        Case "OpenAudio"
          StartAudio()
        Case "OpenISO"
          StartIso()
      End Select
      _initDone = True
    ElseIf ApplicationSettings.StartLastUsedProjectType Then
      If ApplicationSettings.LastUsedProjectType = 1 Then
        StartData()
      Else
        StartAudio()
      End If
    End If

  End Sub

  Private Sub cmdExit_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdExit.Click
    'Closing the application
    Me.Close()
  End Sub

  Private Sub bOpen_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles bOpen.Click
    Dim ofd As OpenFileDialog = New OpenFileDialog()
    ofd.Title = RM.GetString("Open Compilation")
    ofd.CheckFileExists = True
    ofd.ValidateNames = True
    ofd.Filter = RM.GetString("CDBurnerXP compilation files") & " (*.dxp;*.axp;*.iso)|*.dxp;*.axp;*.iso"
    If SafeOpenFileDialog.ShowDialog(ofd, Me, "OpenProject") = System.Windows.Forms.DialogResult.OK Then
      If Path.GetExtension(ofd.FileName) = ".dxp" Then
        g_sAction = "od"
        g_sActionFile = ofd.FileName
        StartData()
      ElseIf Path.GetExtension(ofd.FileName) = ".axp" Then
        g_sAction = "oa"
        g_sActionFile = ofd.FileName
        StartAudio()
      ElseIf Path.GetExtension(ofd.FileName) = ".iso" Then
        g_sAction = "oi"
        g_sActionFile = ofd.FileName
        StartIso()
      Else
      End If
    End If
  End Sub

  Private Sub bOptions_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles bOptions.Click
    Using options As New frmOptions()
      options.ShowDialog(Me)
    End Using
  End Sub

  Private Sub lbSelectProject_ItemChosen(ByVal sender As System.Object, ByVal e As AdvancedListBox.ItemChosenEventArgs) Handles lbSelectProject.ItemChosen
    OnSelect(e.ChosenPanel)
  End Sub

  Sub OnSelect(ByVal panel As ListBoxPanel)
    If panel Is Nothing Then Return

    Try
      Cursor = Cursors.WaitCursor

      If panel Is pnlDataDisc Then
        StartData()
      ElseIf panel Is pnlBurnIso Then
        StartIso()
      ElseIf panel Is pnlAudioDisc Then
        StartAudio()
      ElseIf panel Is pnlEraseDisc Then
        StartErase()
      ElseIf panel Is pnlCopyDisc Then
        StartCopyData()
      ElseIf panel Is pnlVideoDisc Then
        StartVideo()
      Else
        Return
      End If
    Finally
      Cursor = Cursors.Default
    End Try
  End Sub

  Private Sub bOK_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles bOK.Click
    OnSelect(lbSelectProject.SelectedPanel)
  End Sub

  Private Sub StartData()
    ApplicationSettings.LastUsedProjectType = 1

    Dim addFiles As String = g_sActionFile
    If addFiles.Contains("?") Then
      ' Don't let the layout opener get it!
      g_sActionFile = ""
      g_sAction = ""
    End If

    Dim f As New frmDataCompilation()
    If (addFiles.Contains("?")) Then
      f.AddToQueue(addFiles.Split("?"c))
    End If

    TaskBarExtensions.SetMainWindowHandle(f.Handle)
    f.Show()
    Me.Hide()
  End Sub

  Sub StartErase()
    Using f As New EraseDiscDialog(Nothing, Nothing, False)
      f.ShowDialog(Me)
    End Using
  End Sub

  Private Sub StartIso()
    Using f As New frmCDDVDBurnIso(Nothing, g_sActionFile)
      g_sActionFile = String.Empty
      TaskBarExtensions.SetMainWindowHandle(f.Handle)
      f.ShowDialog(Me)
      TaskBarExtensions.SetMainWindowHandle(Me.Handle)
    End Using
  End Sub

  Private Sub StartVideo()
    Using f As New BurnVideoDvdDialog()
      TaskBarExtensions.SetMainWindowHandle(f.Handle)
      f.ShowDialog(Me)
      TaskBarExtensions.SetMainWindowHandle(Me.Handle)
    End Using
  End Sub

  Private Sub StartCopyData()
    Using f As New frmCDDVDCopy()
      TaskBarExtensions.SetMainWindowHandle(f.Handle)
      f.ShowDialog(Me)
      TaskBarExtensions.SetMainWindowHandle(Me.Handle)
    End Using
  End Sub

  Sub StartAudio()
    ApplicationSettings.LastUsedProjectType = 2

    Dim f As New frmAudioCompilation()
    TaskBarExtensions.SetMainWindowHandle(f.Handle)
    f.Show()
    Me.Hide()

  End Sub

#Region "Notify Icon"

  Private Sub ShowFromTray()
    User32.ShowWindow(Me.Handle, 5)
    MediaNotifier.Enabled = True
    Me.BringToFront()
  End Sub

  Private Sub MinimiseToTray()
    User32.ShowWindow(Me.Handle, 0)
  End Sub

  Private Sub cmnuRestore_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmnuRestore.Click
    Me.ntyMain.Visible = False
    ShowFromTray()
    Me.Activate()
  End Sub

  Private Sub cmnuClose_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmnuClose.Click
    m_IgnoreNotify = True
    Me.Close()
  End Sub

  Private Sub ntyMain_MouseDown(ByVal sender As System.Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles ntyMain.MouseDown
    If e.Button = System.Windows.Forms.MouseButtons.Left AndAlso e.Clicks = 2 Then
      ShowFromTray()
    End If
  End Sub

  Protected Overrides Sub WndProc(ByRef m As System.Windows.Forms.Message)
    Select Case m.Msg

      Case User32.WM_CLOSE
        If Not m_IgnoreNotify AndAlso ApplicationSettings.ExitToNotify Then
          MinimiseToTray()
          ntyMain.Visible = True
          'Turn off cd insert notification while not in mdimain
          MediaNotifier.Enabled = False
        Else
          MyBase.WndProc(m)
        End If
      Case Else
        MyBase.WndProc(m)
    End Select
  End Sub

#End Region

End Class
