'*************************************************************************
' Klass: frmCDDVDWriter
' Beroenden: 
' Syfte: 
' F�rfattare: Fredrik Haglund, Stefan Haglund
' Senast �ndrad av: Stefan Haglund
' Senast �ndrad: 2007-04-25 20:42
' Kommentarer: 
'*************************************************************************setting cu

Imports System.IO
Imports System.Threading
Imports System.Globalization
Imports System.Resources
Imports Microsoft.Win32
Imports CDBurnerXP.IO
Imports CDBurnerXP.Windows.Forms
Imports CDBurnerXP.Burning
Imports CDBurnerXP.Controls
Imports CDBurnerXP.Forms
Imports CDBurnerXP.Net
Imports CDBurnerXP

Public Class frmDataCompilation
  Public Shared CDDVDInstance As frmDataCompilation

  Private blnSessionImported As Boolean = False
  Private m_ForceMedia As Single = 0
  Private m_MsgBoxOwner As Form = Me
  Private m_InitDone As Boolean = False 'Prevent form from closing during init
  Private mDevice As NMSDVDXLib.DeviceX
  Private m_AddQueue As New Queue(Of DragDropperParams)
  Private m_CurrentMediaInfo As MediaInformation
  Private WithEvents m_MainToolbar As MainToolStrip
  Private m_ExitApplicationAfterClose As Boolean = True

  Public Overrides ReadOnly Property ExitApplicationAfterClose() As Boolean
    Get
      Return m_ExitApplicationAfterClose
    End Get
  End Property

  'structure holds file info from enuming
  Public Class DragDropperParams
    Public Data() As String
    Public Droppath As String

    Public Sub New(ByVal pData() As String, Optional ByVal pDroppath As String = "")
      Data = pdata
      Droppath = pdroppath
    End Sub
  End Class

  Private Structure MediaInformation
    Dim MediaType As NMSDVDXLib.eMediaTypeX
    Dim MediaUsedSpaceBlocks As Integer
    Dim MediaTotalSpaceBlocks As Integer
    Dim CompilationFiles As Integer
    Dim CompilationDirs As Integer
  End Structure

  Public Property MsgBoxOwner() As Form
    Get
      Return m_MsgBoxOwner
    End Get
    Set(ByVal Value As Form)
      m_MsgBoxOwner = Value
    End Set
  End Property

  Public ReadOnly Property FileLayout() As NMSDVDXLib.IFileLayoutX
    Get
      Return m_FileLayoutManager.DataImage.GetFileLayout()
    End Get
  End Property

  Private Sub MainToolbarClicked(ByVal sender As Object, ByVal e As System.Windows.Forms.ToolStripItemClickedEventArgs) Handles m_MainToolbar.ItemClicked
    If e.ClickedItem.Tag Is Nothing Then Exit Sub

    Select Case e.ClickedItem.Tag.ToString
      Case "Open"
        mnuOpen.PerformClick()
      Case "Save"
        mnuSave.PerformClick()
      Case "SaveAs"
        mnuSaveAs.PerformClick()
      Case "Close"
        m_ExitApplicationAfterClose = False
        Me.Close()
      Case "Print"
        mnuPrintCover.PerformClick()
      Case "Erase"
        mnuEraseDisc.PerformClick()
      Case "Burn"
        mnuWriteDisc.PerformClick()
      Case "Dropbox"
        mnuDropBox.PerformClick()
      Case "DiscInfo"
        mnuDiscInfo.PerformClick()
      Case "RecorderInfo"
        mnuRecorderInfo.PerformClick()
      Case "Eject"
        mnuEjectDisc.PerformClick()
      Case "CloseTray"
        mnuCloseTray.PerformClick()
    End Select
  End Sub

  Public Sub New()
    ' This call is required by the Windows Form Designer.
    InitializeComponent()

    m_MainToolbar = New MainToolStrip()

    Me.Controls.Add(m_MainToolbar)

    tbNumCopies.NumericUpDown.Minimum = 1
    tbNumCopies.NumericUpDown.Maximum = 99
    tbNumCopies.NumericUpDown.TextAlign = HorizontalAlignment.Right
    tbNumCopies.NumericUpDown.Value = Math.Max(1, Math.Min(99, Conversion.ToInt(Settings.GetValue("frmCDDVDBurn", "NumberOfCopies", 1))))

    Me.Text = RM.GetString("Data Compilation")
    filSelect.ShowHiddenItems = PathEx.IsShowHiddenFilesEnabled
    dirSelect.ShowHiddenObjects = PathEx.IsShowHiddenFilesEnabled
    lvFiles.ContextMenu = cmuCompilation
    tvDirectories.ContextMenu = cmuCompilation
    m_FileLayoutManager.SpecialFileHandler = AddressOf HandleSpecialFile
    filSelect.Filter = FileView.FilterType.Data
  End Sub

  Protected Overrides Sub OnClosed(ByVal e As System.EventArgs)
    MyBase.OnClosed(e)

    RemoveHandler cboDrives.DeviceSelect.SelectedIndexChanged, AddressOf OnDriveSelectionChanged

    ApplicationSettings.DataFileSystem = m_FileLayoutManager.DataImage.VolumeType
    ApplicationSettings.DataIsoLevel = m_FileLayoutManager.DataImage.ISOLevel

    If cboDrives.DeviceSelect.SelectedIndex >= 0 Then
      ApplicationSettings.DataLastDeviceUsed = cboDrives.DeviceSelect.SelectedDeviceLetter
    End If
    ApplicationSettings.DataExplorer = mnuExplorer.Checked

    If splVertical.Orientation = Orientation.Vertical Then
      ApplicationSettings.DataTileVertical = True
    Else
      ApplicationSettings.DataTileVertical = False
    End If

    ApplicationSettings.DataSplitterExplorer = splExplorer.SplitterDistance
    ApplicationSettings.DataSplitterLayout = splLayout.SplitterDistance
    ApplicationSettings.DataSplitterVertical = splVertical.SplitterDistance

    If Not filSelect Is Nothing Then
      filSelect.SaveColumnLayout()
      ApplicationSettings.DataExplorerViewStyle = filSelect.ViewStyle
      ApplicationSettings.DataExplorerPath = filSelect.CurrentFolder
    End If

    Try
      ' Try to save settings. Fails if no write permission in user folder. Rare, but shit happens.
      ApplicationSettings.Save()
    Catch ex As Exception
    End Try

    'Clean up cd notify handlers
    MediaNotifier.Enabled = False

    If ExitApplicationAfterClose Then
      frmSelectProject.Instance.Close()
    Else
      frmSelectProject.Instance.Show()
    End If
  End Sub

  Protected Overrides Sub OnClosing(ByVal e As System.ComponentModel.CancelEventArgs)
    MyBase.OnClosing(e)

    If Not m_InitDone Then
      e.Cancel = True
      Exit Sub
    End If

    If m_FileLayoutManager.IsUnsaved Then
      If Not m_FileLayoutManager.IsDataImageEmpty Then
        Dim result As DialogResult = TaskDialog.ShowOptionalMessageBox(Me, "SaveDataCompilation", RM.GetString("Save Compilation"), "", RM.GetString("Your compilation has not been saved. Do you want to save your changes before closing it?"), eTaskDialogButtons.YesNoCancel, eSysIcons.Warning, System.Windows.Forms.DialogResult.No)
        Select Case result
          Case DialogResult.Yes
            mnuSaveAs.PerformClick()
          Case DialogResult.Cancel
            e.Cancel = True
            Exit Sub
        End Select
      End If
    End If
  End Sub

  Protected Overrides Sub OnLoad(ByVal e As System.EventArgs)
    If (DesignMode) Then Exit Sub

    'Language Set
    mnuFile.Text = RM.GetString("&File")
    mnuOpen.Text = RM.GetString("Open...")
    mnuSave.Text = RM.GetString("Save")
    mnuSaveAs.Text = RM.GetString("Save As...")
    mnuSaveISO.Text = RM.GetString("Save Image As ISO File...")
    mnuConverISO.Text = RM.GetString("Convert ISO Image...")
    mnuPrintCover.Text = RM.GetString("Print Cover...")
    mnuOptions.Text = RM.GetString("Op&tions")
    mnuExit.Text = RM.GetString("E&xit")
    mnuEdit.Text = RM.GetString("Edit")
    mnuClearCompilation.Text = RM.GetString("&Clear compilation")
    mnuRecorder.Text = RM.GetString("Recorder")
    mnuEjectDisc.Text = RM.GetString("Eject Disc")
    mnuCloseTray.Text = RM.GetString("Close Tray")
    mnuRecorderInfo.Text = RM.GetString("Device &information")
    mnuReportDrive.Text = RM.GetString("Report Working Drive...")
    mnuFileSystem.Text = RM.GetString("File system")
    mnuDiscInfo.Text = RM.GetString("Disc &information")
    mnuImportSession.Text = RM.GetString("Import Session...")
    mnuSetLabels.Text = RM.GetString("Set Disc Labels...")
    mnuBootOptions.Text = RM.GetString("Boot Options...")
    mnuDiscSpanning.Text = RM.GetString("Disc spa&nning...")
    mnuEraseDisc.Text = RM.GetString("Erase Disc...")
    mnuWriteDisc.Text = RM.GetString("Burn Disc...")
    mnuView.Text = RM.GetString("&View")
    mnuDisc.Text = RM.GetString("D&isc")
    mnuExplorer.Text = RM.GetString("E&xplorer panel")
    mnuTile.Text = RM.GetString("&Tile")
    mnuDropBox.Text = RM.GetString("Show &dropbox")
    mnuTileHorizontal.Text = RM.GetString("&Horizontal")
    mnuTileVertical.Text = RM.GetString("&Vertical")
    mnuHelpOnLine.Text = RM.GetString("&Online help")
    mnuCheckVersion.Text = RM.GetString("&Check for new version")
    mnuWeb.Text = RM.GetString("CDBurnerXP on the &web")
    mnuAbout.Text = RM.GetString("&About CDBurnerXP")
    mnuHelp.Text = RM.GetString("Help")
    mnuClose.Text = RM.GetString("Close")
    mnuNewFolder.Text = RM.GetString("&New Folder")
    mnuAddFiles.Text = RM.GetString("Add files...")
    mnuAddFolder.Text = RM.GetString("Add f&older...")
    mnuFileFilter.Text = RM.GetString("&Filter files...")
    mnuSelectAll.Text = RM.GetString("Select &all")
    mnuInvertSelection.Text = RM.GetString("Invert &selection")
    mnuUpdateCompilation.Text = RM.GetString("&Update compilation")
    lblDriveImage.ToolTipText = RM.GetString("Target device for the burning process")
    cboDrives.ToolTipText = RM.GetString("Target device for the burning process")
    Dim str As String = filSelect.GetColumnName("", 1)
    lvFiles.Columns("cName").Text = RM.GetString("Name")
    lvFiles.Columns("cSize").Text = RM.GetString("Size")
    lvFiles.Columns("cType").Text = RM.GetString("Type")
    lvFiles.Columns("cDate").Text = RM.GetString("Date Modified")
    tbBurn.Text = RM.GetString("Burn")
    tbErase.Text = RM.GetString("Erase")
    tbClear.Text = RM.GetString("Clear")
    tbAdd.Text = RM.GetString("Add")
    tbRemove.Text = RM.GetString("Re&move")
    Sizebar3.WarningText = RM.GetString("Data exceeds the size of the media")
    Sizebar3.UnitText = RM.GetString("MB")
    cmuiUpdateCompilation.Text = RM.GetString("&Update compilation")
    cmuiRemove.Text = RM.GetString("&Remove")
    cmuiRename.Text = RM.GetString("Rena&me") '*
    cmuiSelectAll.Text = RM.GetString("Select &all")
    cmuiPaste.Text = RM.GetString("&Paste")
    cmuiProperties.Text = RM.GetString("&Properties")
    cmuiInvertSelection.Text = RM.GetString("Invert &selection")
    cmuiCreateDirectory.Text = RM.GetString("&New Folder")
    cmuiClear.Text = RM.GetString("&Clear compilation")
    cmnuSizebarForceDiscType.Text = RM.GetString("Select Media Size")
    cmnuSizebarRefreshDisc.Text = RM.GetString("Refresh Disc")
    cmnuSizebarAutoDetect.Text = RM.GetString("&Automatically detect media size")
    cmnuSizebarDVD9.Text = DiscType.DVDDL.Description
    cmnuSizebarDVD5.Text = DiscType.DVD.Description
    cmnuSizebarCD90.Text = DiscType.CD90.Description
    cmnuSizebarCD80.Text = DiscType.CD80.Description
    cmnuSizebarCD74.Text = DiscType.CD74.Description
    cmnuSizebarCD63.Text = DiscType.CD63.Description
    cmnuSizebarCD21.Text = DiscType.CD21.Description
    cmnuSizebarHDDVDDual.Text = DiscType.HDDVDDL.Description
    cmnuSizebarHDDVDSingle.Text = DiscType.HDDVD.Description
    cmnuSizebarBDDual.Text = DiscType.BDDL.Description
    cmnuSizebarBDSingle.Text = DiscType.BD.Description

    strAddingFile = RM.GetString("Adding file: ")
    strAddingFilesFrom = RM.GetString("Adding files from: ")
    strUpdatingComp = RM.GetString("Updating Compilation: ")
    tbLblCopies.Text = RM.GetString("&Number of copies:")
    CDDVDInstance = Me

    m_FileLayoutManager.Clear(DataCompilation.ReplaceDiscLabelVariables(ApplicationSettings.DataLabel))

    If ApplicationSettings.DataFileSystem <> -1 Then
      m_FileLayoutManager.DataImage.VolumeType = CType(ApplicationSettings.DataFileSystem, NMSDVDXLib.eVolumeTypeX)
    End If

    Select Case ApplicationSettings.DataIsoLevel
      Case NMSDVDXLib.eISOLevelX.isoLevel01
        m_FileLayoutManager.DataImage.ISOLevel = NMSDVDXLib.eISOLevelX.isoLevel01
      Case NMSDVDXLib.eISOLevelX.iso9660_1999
        m_FileLayoutManager.DataImage.ISOLevel = NMSDVDXLib.eISOLevelX.iso9660_1999
      Case Else
        m_FileLayoutManager.DataImage.ISOLevel = NMSDVDXLib.eISOLevelX.isoLevel02
    End Select

    RecalcImageSize()

    m_InitDone = True

    dirSelect.FastModeSynchronization = True
    filSelect.CurrentFolder = ApplicationSettings.DataExplorerPath
    dirSelect.FastModeSynchronization = False
    filSelect.ViewStyle = CType(ApplicationSettings.DataExplorerViewStyle, LogicNP.FileViewControl.ViewStyles)

    If ApplicationSettings.DataExplorer = False Then
      mnuExplorer.PerformClick()
    End If

    If ApplicationSettings.DataTileVertical = True Then
      splVertical.Orientation = Orientation.Vertical
      mnuTileHorizontal.Checked = False
      mnuTileVertical.Checked = True
    Else
      splVertical.Orientation = Orientation.Horizontal
      mnuTileHorizontal.Checked = True
      mnuTileVertical.Checked = False
    End If

    If Not AutoUpdate.IsOnlineUpdateAllowed Then
      mnuCheckVersion.Visible = False
      mnuSep9.Visible = False
    End If

    ' Set correct drive
    If cboDrives.DeviceSelect.SelectedDeviceLetter <> ApplicationSettings.DataLastDeviceUsed Then
      cboDrives.DeviceSelect.SelectedDeviceLetter = ApplicationSettings.DataLastDeviceUsed
    End If
    ' Ensure update
    OnDriveSelectionChanged(Me, Nothing)
    ' Attach event
    AddHandler cboDrives.DeviceSelect.SelectedIndexChanged, AddressOf OnDriveSelectionChanged

    Me.Activate()

    MyBase.OnLoad(e)

    AdjustSplitters()
  End Sub

  Protected Overrides Sub OnLoaded(ByVal sender As Object, ByVal e As System.EventArgs)
    If g_sAction <> "" Then
      LoadCompilation(g_sActionFile)

      g_sAction = ""
      g_sActionFile = ""
    End If
  End Sub

  Private Sub PreventSystemChanges(ByVal sender As Object, ByVal e As LogicNP.FolderViewControl.FolderViewDragOverEventArgs) Handles dirSelect.FolderViewDragOver
    If ApplicationSettings.PreventDefaultDragDrop Then
      e.Effects = DragDropEffects.None
    End If
  End Sub

  Private Sub PreventSystemChanges(ByVal sender As Object, ByVal e As LogicNP.FolderViewControl.FolderViewDragDropEventArgs) Handles dirSelect.FolderViewDragDrop
    e.DoDefaultShellDropAction = Not ApplicationSettings.PreventDefaultDragDrop
  End Sub

  Private Sub OnDriveSelectionChanged(ByVal sender As Object, ByVal e As System.EventArgs)
    If cboDrives.DeviceSelect.SelectedIndex < 0 Then Exit Sub

    If mDevice IsNot Nothing AndAlso mDevice.GetDriveLetter() = cboDrives.DeviceSelect.SelectedDeviceLetter Then Exit Sub

    If blnSessionImported Then
      'We switched drive from which we had an imported session
      If MessageBox.Show(Me, RM.GetString("Changing device while having an imported session will clear the current compilation. Do you want to continue?"), Application.ProductName, MessageBoxButtons.YesNo, MessageBoxIcon.Question, MessageBoxDefaultButton.Button2) = System.Windows.Forms.DialogResult.No Then
        Exit Sub
      Else
        m_FileLayoutManager.Clear()
        blnSessionImported = False
      End If
    End If

    mDevice = cboDrives.DeviceSelect.SelectedDevice
    'Set cd notify to only check the current used drive and start it
    MediaNotifier.DriveLetter = cboDrives.DeviceSelect.SelectedDeviceLetter
    MediaNotifier.Enabled = True

    ThreadPool.QueueUserWorkItem(New WaitCallback(AddressOf ThreadUpdateMediaInfo))
  End Sub

  Private Sub ThreadUpdateMediaInfo()
    RefreshMediaInfo()
    Try
      Me.Invoke(New MethodInvoker(AddressOf RecalcImageSize))
      Me.Invoke(New MethodInvoker(AddressOf AutoImportSession))
    Catch ex As InvalidOperationException
    End Try
  End Sub

  Private Sub DisplayCurrentMediaType(ByVal mType As NMSDVDXLib.eMediaTypeX)
    Dim onDiscText As String = String.Empty

    onDiscText = BurningEngine.GetDiscDescription(mType)
    If mType = NMSDVDXLib.eMediaTypeX.mtNOT_LOADED Then
      sbrInDrive.Image = My.Resources.DiscMissing
    Else
      sbrInDrive.Image = My.Resources.CD

      Dim usedBytes As Long = FormatFileSize.BlocksToDataBytes(m_CurrentMediaInfo.MediaUsedSpaceBlocks)
      If usedBytes = 0 Then
        onDiscText += " (" & RM.GetString("Empty") & ")"
      Else
        onDiscText += String.Format(" " & RM.GetString("(Used: {0})"), FormatFileSize.Format(usedBytes))
      End If

    End If

    sbrInDrive.Text = onDiscText
  End Sub

  Private Sub RefreshMediaInfo()
    If mDevice Is Nothing Then Exit Sub

    mDevice.Refresh(True)
    m_CurrentMediaInfo.MediaType = mDevice.GetDiscInfo.GetMediaType
    m_CurrentMediaInfo.MediaTotalSpaceBlocks = mDevice.GetCapacityBlocks(True)
    m_CurrentMediaInfo.MediaUsedSpaceBlocks = mDevice.GetUsedSpaceBlocks(True)
    m_CurrentMediaInfo.CompilationFiles = FileLayout.GetFileCount
    m_CurrentMediaInfo.CompilationDirs = FileLayout.GetFolderCount
  End Sub

  Public Sub RecalcImageSize()
    Dim lngSizeBlocks As Long
    Dim dblsizebytes As Double
    Dim DiscType As NMSDVDXLib.eMediaTypeX = m_CurrentMediaInfo.MediaType
    DisplayCurrentMediaType(DiscType)

    lngSizeBlocks = m_FileLayoutManager.DataImage.GetImageSizeBlocks
    dblsizebytes = m_FileLayoutManager.DataImage.GetImageSizeBytes

    If DiscType = NMSDVDXLib.eMediaTypeX.mtNOT_LOADED Then
      Sizebar3.OnDiscValue = 0
      If m_ForceMedia = 0 Then
        Sizebar3.OverDiscValue = 500000
      Else
        Sizebar3.OverDiscValue = m_ForceMedia
      End If
    Else
      Sizebar3.OnDiscValue = CSng((m_CurrentMediaInfo.MediaUsedSpaceBlocks / 1024 / 1024) * 2048)
      If m_ForceMedia = 0 Then
        Sizebar3.OverDiscValue = CSng((m_CurrentMediaInfo.MediaTotalSpaceBlocks / 1024 / 1024) * 2048)
      Else
        Sizebar3.OverDiscValue = m_ForceMedia
      End If

    End If
    If m_FileLayoutManager.DataImage.GetFileLayout().GetFileCount() > 0 Then
      Sizebar3.BarValue = CSng(dblsizebytes / 1048576)
    Else
      ' don't confuse user
      Sizebar3.BarValue = 0
    End If
    If DiscType = NMSDVDXLib.eMediaTypeX.mtNOT_LOADED And m_ForceMedia = 0 Then
      Sizebar3.MaxValue = CSng(dblsizebytes / 1048576 + 50)
    Else
      Sizebar3.MaxValue = CSng(IIf(Sizebar3.OnDiscValue + Sizebar3.BarValue < Sizebar3.OverDiscValue, Sizebar3.OverDiscValue + 0.15 * Sizebar3.OverDiscValue, (Sizebar3.OnDiscValue + Sizebar3.BarValue) + 0.15 * (Sizebar3.OnDiscValue + Sizebar3.BarValue)))
    End If

    If Sizebar3.MaxValue = 0 Then
      Sizebar3.MaxValue = CSng(dblsizebytes / 1048576 + 50)
    End If

    Dim fileSystem As String
    Select Case m_FileLayoutManager.DataImage.VolumeType
      Case NMSDVDXLib.eVolumeTypeX.vtISO9660
        fileSystem = "ISO 9660"
      Case NMSDVDXLib.eVolumeTypeX.vtISO9660_Joliet_UDF
        fileSystem = "ISO 9660/Joliet/UDF"
      Case NMSDVDXLib.eVolumeTypeX.vtISO9660_UDF
        fileSystem = "ISO 9660/UDF"
      Case NMSDVDXLib.eVolumeTypeX.vtUDF
        fileSystem = "UDF"
      Case Else
        fileSystem = "ISO 9660/Joliet"
    End Select

    Dim strStatusText As String
    strStatusText = String.Format(RM.GetString(" {0} with {1} folders and {2} files"), fileSystem, FileLayout.GetFolderCount, FileLayout.GetFileCount)

    sbrISOFormat.Text = strStatusText

    strStatusText = String.Format(RM.GetString(" Total Estimated Size: {0} MB "), ((dblsizebytes / 1048576) + Sizebar3.OnDiscValue).ToString("0.00"))

    sbrEstimated.Text = strStatusText
    If m_CurrentMediaInfo.MediaType <> NMSDVDXLib.eMediaTypeX.mtNOT_LOADED Then
      Dim RemainingColor As Color
      If m_ForceMedia = 0 Then
        If ((m_CurrentMediaInfo.MediaTotalSpaceBlocks / 1048576 * 2048) - (dblsizebytes / 1048576) - Sizebar3.OnDiscValue) < 0 Then
          RemainingColor = Color.Red
        Else
          RemainingColor = Color.Black
        End If
        strStatusText = String.Format(RM.GetString(" Remaining Size: {0} MB "), ((m_CurrentMediaInfo.MediaTotalSpaceBlocks / 1048576 * 2048) - (dblsizebytes / 1048576) - Sizebar3.OnDiscValue).ToString("0.00"))
      Else
        If (m_ForceMedia - (dblsizebytes / 1048576) - Sizebar3.OnDiscValue) < 0 Then
          RemainingColor = Color.Red
        Else
          RemainingColor = Color.Black
        End If
        strStatusText = String.Format(RM.GetString(" Remaining Size: {0} MB "), (m_ForceMedia - (dblsizebytes / 1048576) - Sizebar3.OnDiscValue).ToString("0.00"))
      End If
      sbrRemaining.Text = strStatusText
    Else
      If m_ForceMedia = 0 Then
        strStatusText = RM.GetString(" Remaining Size: N/A ")
        sbrRemaining.Text = strStatusText
      Else
        Dim RemainingColor As Color
        If (m_ForceMedia - (dblsizebytes / 1048576) - Sizebar3.OnDiscValue) < 0 Then
          RemainingColor = Color.Red
        Else
          RemainingColor = Color.Black
        End If
        strStatusText = String.Format(RM.GetString(" Remaining Size: {0} MB "), (m_ForceMedia - (dblsizebytes / 1048576) - Sizebar3.OnDiscValue).ToString("0.00"))
        sbrRemaining.Text = strStatusText
      End If
    End If

    If String.IsNullOrEmpty(sbrRemaining.Text) Then
      Sizebar3.AccessibleDescription = sbrEstimated.Text.Trim()
    Else
      Sizebar3.AccessibleDescription = sbrEstimated.Text.Trim() & ", " & sbrRemaining.Text
    End If
    Sizebar3.UpdateSizeBar()
  End Sub

  Private Sub AutoImportSession()
    If m_AutoImportEnabled = False Then Exit Sub
    If blnSessionImported Then Exit Sub
    If m_CurrentMediaInfo.MediaType = NMSDVDXLib.eMediaTypeX.mtNOT_LOADED Then Exit Sub
    If Not mDevice.GetDiscInfo.IsOpen Then Exit Sub
    'Is there anything to import ie a valid file system?
    If mDevice.GetLogicalVolumeTypes(0) = 0 Then Exit Sub
    'There is a session to import

    Dim dialogRes As TaskDialogResult = TaskDialog.ShowCommandBox(Me, Application.ProductName, _
                RM.GetString("Do you want to continue the inserted disc?"), _
                RM.GetString("The inserted disc contains one or more sessions. You can import their data to continue the disc and add, delete or update files."), _
                "", RM.GetString("Regardless of your choice, the current disc's contents remain unchanged until the burning process starts."), "", _
                RM.GetString("Continue disc\nAllows you to add, delete and update existing files on the disc.|Ignore previous sessions\nThe data currently on disc will not be available anymore after the burning process completes.").Replace("\n", ControlChars.Lf), _
                True, _
                eSysIcons.Question, _
                eSysIcons.Information)


    If dialogRes.SelectedCommand = 0 Then
      If m_FileLayoutManager.ImportFromDisc(mDevice) Then
        blnSessionImported = True
      Else
        Dim strImportError As String = RM.GetString(BurningEngine.GetImportErrorMsg(m_FileLayoutManager.DataImage.GetImportError))
        MessageBox.Show(m_MsgBoxOwner, RM.GetString("Session could not be imported.") & " " & strImportError, Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Error)
        m_FileLayoutManager.Clear()
      End If
    End If
  End Sub

#Region "Drag'nDrop"

  Public Sub AddToQueue(ByVal data() As String, Optional ByVal droppath As String = "")
    ' Check previously added item - if duplicate: ignore; if same params: merge jobs
    If m_AddQueue.Count > 0 Then
      Dim prevParams As DragDropperParams = m_AddQueue.Peek()
      If prevParams.Droppath = droppath Then
        Dim listOld As New List(Of String)(prevParams.Data)
        For Each newPath As String In data
          If Not listOld.Contains(newPath) Then
            listOld.Add(newPath)
          End If
        Next
        prevParams.Data = listOld.ToArray()
        Exit Sub
      End If
    End If

    m_AddQueue.Enqueue(New DragDropperParams(data, droppath))
  End Sub

  Private Sub tmrQueue_Tick(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles tmrQueue.Tick
    While m_AddQueue.Count > 0
      tmrQueue.Stop()
      Dim params As DragDropperParams = m_AddQueue.Dequeue()
      AddFilesToCompilation(params.Data, params.Droppath)
      tmrQueue.Start()
    End While
  End Sub


  Private Sub AddFilesToCompilation(ByVal Data() As String, Optional ByVal Droppath As String = "")
    m_FileLayoutManager.BeginAddPaths(Droppath, Data, MsgBoxOwner)
  End Sub

#End Region

  Private Sub LoadCompilation(ByVal strFileName As String)
    If String.IsNullOrEmpty(strFileName) Then Exit Sub

    Try
      m_FileLayoutManager.LoadDataCompilation(strFileName)

      If ApplicationSettings.DataUpdateAutomatically Then
        cmuiUpdateCompilation.PerformClick()
      End If
    Catch ex As Exception
      MessageBox.Show(RM.GetString("An error occured while loading the file:" & " ") & ex.Message, Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Error)
    End Try

  End Sub

  Private Sub ShowFileSelectionDialog()
    ' user wants to add a file, but didn't select any. we need to help.
    Using dialog As New OpenFileDialog
      dialog.Multiselect = True
      dialog.InitialDirectory = filSelect.CurrentFolder
      dialog.Filter = "*.*|*.*"
      If SafeOpenFileDialog.ShowDialog(dialog, Me) = System.Windows.Forms.DialogResult.OK Then
        AddToQueue(dialog.FileNames, tvDirectories.GetSelectedPath())
      End If
      Exit Sub
    End Using
  End Sub

#Region "Explorer Toolbar"

  Private Sub tbFilter_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles mnuFileFilter.Click
    Using f As New frmFilter(True)
      If f.ShowDialog(Me) = System.Windows.Forms.DialogResult.OK Then
        filSelect.RefreshView()
      End If
    End Using
  End Sub

  Private Sub tbNewFolder_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles mnuNewFolder.Click
    m_FileLayoutManager.CreateNewDirectory()
  End Sub

  Private Sub tbSelectAll_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles mnuSelectAll.Click
    If lvFiles.Focused Then
      lvFiles.SelectAll()
    Else
      Dim lstItm As LogicNP.FileViewControl.ListItem
      For i As Integer = 1 To filSelect.ItemCount
        lstItm = filSelect.GetListItem(i)
        lstItm.Selected = True
      Next
    End If
  End Sub

#End Region

#Region "Mid Toolbar"
  Private Sub tbBurn_ButtonClick(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles tbBurn.Click

    If m_FileLayoutManager.DataImage.GetFileLayout.GetFileCount() = 0 Then
      If MessageBox.Show(RM.GetString("You did not add any files to the compilation. Do you want to burn the disc anyway?"), Application.ProductName, MessageBoxButtons.YesNo, MessageBoxIcon.Warning, MessageBoxDefaultButton.Button2) = System.Windows.Forms.DialogResult.No Then
        Exit Sub
      End If
    End If

    If Not m_FileLayoutManager.ShowFileNameChanges(Me) Then
      Exit Sub
    End If

    Dim burnOption As DataQuickBurnOption = DataQuickBurnOption.AdvancedSettings

    If ApplicationSettings.DataAskForSettings Then
      Dim dialogRes As TaskDialogResult = TaskDialog.ShowCommandBox(Me, RM.GetString("Burn Disc"), _
                                              RM.GetString("How do you want to burn your disc?"), _
                                              RM.GetString("If you do not choose to see the advanced settings, CDBurnerXP will set the remaining options for you automatically."), _
                                              "", _
                                              "", _
                                              RM.GetString("Don't ask me again"), _
                                              RM.GetString("Leave disc open\nAllows you to add more data at a later time|Finalise disc\nNo more data can be added later|Let me choose advanced settings").Replace("\n", ControlChars.Lf), _
                                              True, _
                                              eSysIcons.Question, _
                                              eSysIcons.Question)
      If dialogRes.DialogResult = DialogResult.Cancel Then
        Return
      End If
      If dialogRes.VerificationChecked Then
        ApplicationSettings.DataAskForSettings = False
      End If
      burnOption = CType(dialogRes.SelectedCommand, DataQuickBurnOption)
    End If

    ' while waiting for the right disc, no import please!
    m_AutoImportEnabled = False
    Dim discSpanningSelected As Boolean
    If CheckDisc.ShowWithDiscSpanning(Me, mDevice, CheckDisc.DiscFeature.Writable, CheckDisc.DiscType.Any, CLng(m_FileLayoutManager.DataImage.GetImageSizeBytes()), blnSessionImported, True, discSpanningSelected) Then
      Dim f As New frmCDDVDBurn(m_FileLayoutManager, mDevice, burnOption)
      f.NumberOfCopies = CInt(tbNumCopies.NumericUpDown.Value)
      Me.SwitchToWindow(f)

      If m_CurrentMediaInfo.MediaType <> NMSDVDXLib.eMediaTypeX.mtNOT_LOADED Then
        RefreshMediaInfo()
        RecalcImageSize()
      End If
    ElseIf discSpanningSelected Then
      mnuDiscSpanning.PerformClick()
    End If

    m_AutoImportEnabled = True
  End Sub

  Private Sub tbErase_ButtonClick(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles tbErase.Click
    Using f As New EraseDiscDialog(If(mDevice Is Nothing, Nothing, mDevice.GetDriveLetter()), m_FileLayoutManager, blnSessionImported)
      f.ShowDialog(Me)
    End Using

    If m_CurrentMediaInfo.MediaType <> NMSDVDXLib.eMediaTypeX.mtNOT_LOADED Then
      RefreshMediaInfo()
      RecalcImageSize()
    End If
  End Sub

  Private Sub tbClear_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles tbClear.Click
    mnuClearCompilation.PerformClick()
  End Sub

  Private Sub tbAdd_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles tbAdd.Click
    If filSelect.SelectedCount = 0 Then
      ShowFileSelectionDialog()
    End If

    Dim i As Integer, k As Integer
    Dim strArray(filSelect.SelectedCount - 1) As String
    For i = 0 To filSelect.ItemCount - 1
      If filSelect.GetListItem(i + 1).Selected Then
        strArray(k) = filSelect.GetListItem(i + 1).Path
        k += 1
      End If
    Next

    AddToQueue(strArray, tvDirectories.GetSelectedPath())
  End Sub

  Private Function HandleSpecialFile(ByVal sourcePaths() As String) As Boolean
    ' Is there something to handle? And if so, handle it?
    If sourcePaths Is Nothing OrElse sourcePaths.Length <> 1 Then Return False

    Dim myFile As String = sourcePaths(0)

    If Path.GetExtension(myFile).ToLower() = ".iso" Then
      If TaskDialog.ShowOptionalMessageBox(MsgBoxOwner, "AddISO", Application.ProductName, "", RM.GetString("You have added an ISO-file to the compilation. Do you want to burn a disc from this ISO-file?"), eTaskDialogButtons.YesNo, eSysIcons.Question, 0) = System.Windows.Forms.DialogResult.Yes Then
        Using f As New frmCDDVDBurnIso(If(mDevice Is Nothing, Nothing, mDevice.GetDriveLetter()), myFile)
          f.ShowDialog(MsgBoxOwner)
          Return True
        End Using
      End If
    ElseIf Playlists.IsSupportedPlaylist(myFile) Then
      If TaskDialog.ShowOptionalMessageBox(MsgBoxOwner, "AddPlaylist", Application.ProductName, "", RM.GetString("You have added a playlist to the compilation. Do you want to add the files within the playlist instead?"), eTaskDialogButtons.YesNo, eSysIcons.Question, 0) = System.Windows.Forms.DialogResult.Yes Then
        AddToQueue(Playlists.GetFilesFromPlaylist(myFile), tvDirectories.GetSelectedPath())
        Return True
      End If
    End If

    Return False
  End Function

  Private Sub tbRemove_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles tbRemove.Click
    DeleteSelection()
  End Sub

#End Region

#Region "Main Menu"

  Private Sub mnuClose_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuClose.Click
    m_ExitApplicationAfterClose = False
    Me.Close()
  End Sub

  Private Sub mnuOpen_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuOpen.Click
    Dim blnBusyFiles As Boolean = False
    If Not m_FileLayoutManager.IsDataImageEmpty Then
      If MessageBox.Show(Me, RM.GetString("Opening an old compilation from file will clear the current compilation. Do you still want to load a compilation?"), "CDBurnerXP", MessageBoxButtons.YesNo, MessageBoxIcon.Warning, MessageBoxDefaultButton.Button1) = System.Windows.Forms.DialogResult.No Then
        Exit Sub
      End If
    End If

    OpenLayoutDialog.Title = RM.GetString("Open Compilation")
    OpenLayoutDialog.Filter = RM.GetString("CDBurnerXP compilation files") & " (*.dxp)|*.dxp"

    If SafeOpenFileDialog.ShowDialog(OpenLayoutDialog, Me, "OpenDataCompilation") = System.Windows.Forms.DialogResult.Cancel Then
      Exit Sub
    End If


    LoadCompilation(OpenLayoutDialog.FileName)
  End Sub

  Private Sub mnuSave_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles mnuSave.Click
    If String.IsNullOrEmpty(m_FileLayoutManager.SavePath) Then
      mnuSaveAs.PerformClick()
    Else
      SaveCompilation(m_FileLayoutManager.SavePath)
    End If
  End Sub

  Private Sub mnuSaveAs_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles mnuSaveAs.Click
    m_SaveLayoutDialog.Title = RM.GetString("Save Compilation")
    m_SaveLayoutDialog.DefaultExt = "dxp"
    m_SaveLayoutDialog.Filter = RM.GetString("CDBurnerXP compilation files") & " (*.dxp)|*.dxp|" & RM.GetString("All files") & " (*.*)|*.*"
    m_SaveLayoutDialog.InitialDirectory = ApplicationSettings.DataSaveCompilationPath
    If m_SaveLayoutDialog.ShowDialog(Me) = System.Windows.Forms.DialogResult.Cancel Then
      Exit Sub
    End If
    Application.DoEvents()
    ApplicationSettings.DataSaveCompilationPath = Path.GetDirectoryName(m_SaveLayoutDialog.FileName)
    strCompilationFilePath = m_SaveLayoutDialog.FileName

    SaveCompilation(m_SaveLayoutDialog.FileName)
  End Sub

  Public Sub SaveCompilation(ByVal filename As String)
    Try
      m_FileLayoutManager.SaveDataCompilation(filename)
    Catch ex As Exception
      MessageBox.Show(RM.GetString("An error occured while saving the file:" + " ") & ex.Message, Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Error)
    End Try

    sbrCompilationInfo.Refresh()
  End Sub

  Private Sub mnuClearCompilation_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles mnuClearCompilation.Click
    If TaskDialog.ShowOptionalMessageBox(Me, "ClearData", RM.GetString("&Clear compilation"), "", RM.GetString("Do you really want to clear the current compilation?"), eTaskDialogButtons.YesNo, eSysIcons.Warning, System.Windows.Forms.DialogResult.Yes) <> System.Windows.Forms.DialogResult.Yes Then
      Exit Sub
    End If

    m_FileLayoutManager.Clear(DataCompilation.ReplaceDiscLabelVariables(ApplicationSettings.DataLabel))

    blnSessionImported = False
  End Sub

  Private Sub mnuSaveISO_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuSaveISO.Click
    Dim dialog As New SaveFileDialog()
    dialog.Title = RM.GetString("Save As ISO File")
    dialog.InitialDirectory = ApplicationSettings.DataSaveISOPath
    dialog.FileName = m_FileLayoutManager.LayoutName & ".iso"
    dialog.Filter = RM.GetString("ISO File") & " (*.iso)|*.iso"
    If dialog.ShowDialog(Me) = DialogResult.OK Then
      ApplicationSettings.DataSaveISOPath = dialog.FileName
      If Not m_FileLayoutManager.SaveAsIso(dialog.FileName) Then
        MessageBox.Show(Me, RM.GetString("Failed creating the ISO file."), Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Error)
      End If
    End If
  End Sub

  Private Sub mnuWriteDisc_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuWriteDisc.Click
    tbBurn.PerformClick()
  End Sub

  Private Sub mnuConverISO_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuConverISO.Click
    'Convert ISO
    Dim frmConvertIso As New frmCDDVDConvertISO
    frmConvertIso.ShowDialog(Me)
    Application.DoEvents()
  End Sub

  Private Sub mnuPrintCover_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuPrintCover.Click
    Using f As New frmPrintCover(m_FileLayoutManager.LayoutName)
      f.CoverPrintDocument.FilesAndFolders.AddRange(lvFiles.GetCoverFileList())
      f.ShowDialog(Me)
    End Using
  End Sub

  Private Sub mnuOptions_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuOptions.Click
    Dim oFrmOptions As New frmOptions()
    oFrmOptions.ShowDialog(Me)
  End Sub

  Private Sub mnuExit_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuExit.Click
    Me.Close()
  End Sub

  Private Sub mnuEjectDisc_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuEjectDisc.Click
    If mDevice IsNot Nothing Then mDevice.Eject()
    RefreshMediaInfo()
    RecalcImageSize()
  End Sub

  Private Sub mnuCloseTray_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuCloseTray.Click
    If mDevice IsNot Nothing Then mDevice.Load()
    RefreshMediaInfo()
    RecalcImageSize()
  End Sub

  Private Sub mnuRecorderInfo_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuRecorderInfo.Click
    'Recorder Information
    Dim frmRecInfo As New frmRecorderInfo(mDevice)
    frmRecInfo.ShowDialog(Me)
    Application.DoEvents()
  End Sub

  Private Sub mnuDiscInfo_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuDiscInfo.Click
    Using dialog As New DiscInfoDialog()
      dialog.DriveLetter = If(mDevice Is Nothing, Nothing, mDevice.GetDriveLetter())
      dialog.ShowDialog(Me)
    End Using
  End Sub

  Private Sub mnuSetLabels_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuSetLabels.Click
    Dim frmLabels As New frmCDDVDLabel(m_FileLayoutManager)
    frmLabels.ShowDialog(Me)
    Application.DoEvents()
  End Sub

  Private Sub mnuBootOptions_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuBootOptions.Click
    Dim frmBoot As New frmCDDVDBootOptions(m_FileLayoutManager)
    frmBoot.ShowDialog(Me)
  End Sub

  Private Sub mnuEraseDisc_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuEraseDisc.Click
    Using f As New EraseDiscDialog(If(mDevice Is Nothing, Nothing, mDevice.GetDriveLetter()), m_FileLayoutManager, blnSessionImported)
      f.ShowDialog(Me)
    End Using

    RefreshMediaInfo()
    RecalcImageSize()
  End Sub

  Private Sub mnuHelpOnLine_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuHelpOnLine.Click
    Try
      System.Diagnostics.Process.Start("http://www.cdburnerxp.se/help/")
    Catch
    End Try
  End Sub

  Private Sub mnuCheckVersion_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuCheckVersion.Click
    AutoUpdate.CheckUpdateOnWebsite()
  End Sub

  Private Sub mnuWeb_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuWeb.Click
    Dim oFrmWeb As New frmWeb
    oFrmWeb.ShowDialog(Me)
  End Sub

  Private Sub mnuAbout_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuAbout.Click
    Dim oFrmAbout As New frmAbout
    oFrmAbout.ShowDialog(Me)
  End Sub

  Private Sub mnuExplorer_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuExplorer.Click
    If mnuExplorer.Checked Then
      mnuExplorer.Checked = False
      splVertical.Panel1Collapsed = True
    Else
      mnuExplorer.Checked = True
      splVertical.Panel1Collapsed = False
    End If
  End Sub

  Private Sub mnuReportDrive_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuReportDrive.Click
    Dim f As New frmReportDrive()
    f.ShowDialog(Me)
  End Sub

  Private Sub mnuTileHorizontal_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuTileHorizontal.Click
    splVertical.Orientation = Orientation.Horizontal
    AdjustSplitters()
    mnuTileHorizontal.Checked = True
    mnuTileVertical.Checked = False
  End Sub

  Private Sub mnuTileVertical_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuTileVertical.Click
    splVertical.Orientation = Orientation.Vertical
    AdjustSplitters()
    mnuTileHorizontal.Checked = False
    mnuTileVertical.Checked = True
  End Sub

  Private Sub mnuDropBox_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuDropBox.Click
    Dim f As New frmCDDVDDropBox(mDevice, m_FileLayoutManager.DataImage, Me)
    Me.Visible = False
    MsgBoxOwner = f
    f.ShowDialog(Me)
    Me.Visible = True
  End Sub

  Private Sub mnuImportSession_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuImportSession.Click
    If Not m_FileLayoutManager.IsDataImageEmpty Then
      If MessageBox.Show(Me, RM.GetString("Importing a session will clear the current file layout. Do you want to continue?"), "CDBurnerXP", MessageBoxButtons.YesNo, MessageBoxIcon.Warning, MessageBoxDefaultButton.Button2) = System.Windows.Forms.DialogResult.No Then Exit Sub
    End If
    Using f As New frmCDDVDImportSession(m_FileLayoutManager, mDevice)
      f.ShowDialog(Me)
      blnSessionImported = f.IsImported
    End Using

    RecalcImageSize()
  End Sub

  Private Sub mnuSplit_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuDiscSpanning.Click
    Using dialog As New SplitDiscDialog()
      dialog.DataCompilation = m_FileLayoutManager.GetDataCompilation()
      If dialog.ShowDialog(Me) = System.Windows.Forms.DialogResult.OK AndAlso Not dialog.ResultDataImages Is Nothing Then
        ' Start burning dialog
        Using f As New frmCDDVDBurn(m_FileLayoutManager, mDevice, DataQuickBurnOption.AdvancedSettings, dialog.ResultDataImages)
          f.NumberOfCopies = CInt(tbNumCopies.NumericUpDown.Value)
          Me.SwitchToWindow(f)

          If m_CurrentMediaInfo.MediaType <> NMSDVDXLib.eMediaTypeX.mtNOT_LOADED Then
            RefreshMediaInfo()
            RecalcImageSize()
          End If
        End Using
      End If
    End Using
  End Sub

#End Region

  Protected Overrides Sub OnMediaInserted(ByVal VolumeLabel As String)
    RefreshMediaInfo()
    RecalcImageSize()
    AutoImportSession()
  End Sub

  Protected Overrides Sub OnMediaRemoved(ByVal VolumeLabel As String)
    If blnSessionImported Then
      blnSessionImported = False
      m_FileLayoutManager.Clear()
    End If
    RefreshMediaInfo()
    RecalcImageSize()
  End Sub

  Private Sub cmnuSizebarAutoDetect_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmnuSizebarAutoDetect.Click
    m_ForceMedia = 0
    RecalcImageSize()
  End Sub

  Private Sub cmnuSizebarHDDVDSingle_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmnuSizebarHDDVDSingle.Click
    m_ForceMedia = 14305
    RecalcImageSize()
  End Sub

  Private Sub cmnuSizebarHDDVDDual_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmnuSizebarHDDVDDual.Click
    m_ForceMedia = 28610
    RecalcImageSize()
  End Sub

  Private Sub cmnuSizebarBDSingle_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmnuSizebarBDSingle.Click
    m_ForceMedia = 23842
    RecalcImageSize()
  End Sub

  Private Sub cmnuSizebarBDDual_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmnuSizebarBDDual.Click
    m_ForceMedia = 47684
    RecalcImageSize()
  End Sub

  Private Sub cmnuCD21_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmnuSizebarCD21.Click
    m_ForceMedia = 184.6
    RecalcImageSize()
  End Sub

  Private Sub cmnuCD63_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmnuSizebarCD63.Click
    m_ForceMedia = 553.7
    RecalcImageSize()
  End Sub

  Private Sub cmnuCD74_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmnuSizebarCD74.Click
    m_ForceMedia = 650.3
    RecalcImageSize()
  End Sub

  Private Sub cmnuCD80_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmnuSizebarCD80.Click
    m_ForceMedia = 703.1
    RecalcImageSize()
  End Sub

  Private Sub cmnuCD90_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmnuSizebarCD90.Click
    m_ForceMedia = 791.0
    RecalcImageSize()
  End Sub

  Private Sub cmnuDVD5_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmnuSizebarDVD5.Click
    m_ForceMedia = 4474
    RecalcImageSize()
  End Sub

  Private Sub cmnuDVD9_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmnuSizebarDVD9.Click
    m_ForceMedia = 8140
    RecalcImageSize()
  End Sub

  Private Sub cmnuSizebarRefreshDisc_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmnuSizebarRefreshDisc.Click
    RefreshMediaInfo()
    RecalcImageSize()
    AutoImportSession()
  End Sub

  Private Sub tmrAutoImportWait_Tick(ByVal sender As Object, ByVal e As System.EventArgs) Handles tmrAutoImportWait.Tick
    If m_InitDone Then
      tmrAutoImportWait.Stop()
      AutoImportSession()
    End If
  End Sub

  Private Sub AdjustSplitters()

    Try
      If ApplicationSettings.DataSplitterExplorer > 0 AndAlso ApplicationSettings.DataSplitterLayout > 0 AndAlso ApplicationSettings.DataSplitterVertical > 0 Then
        splVertical.SplitterDistance = ApplicationSettings.DataSplitterVertical
        splExplorer.SplitterDistance = ApplicationSettings.DataSplitterExplorer
        splLayout.SplitterDistance = ApplicationSettings.DataSplitterLayout
      Else
        ' Use default settings
        If splVertical.Orientation = Orientation.Horizontal Then
          splVertical.SplitterDistance = CInt(Me.Height * 0.4)
          splExplorer.SplitterDistance = CInt(Me.Width * 0.3)
          splLayout.SplitterDistance = CInt(Me.Width * 0.3)
        Else
          splVertical.SplitterDistance = CInt(Me.Width * 0.45)
          splExplorer.SplitterDistance = CInt(Me.Width * 0.2)
          splLayout.SplitterDistance = CInt(Me.Width * 0.2)
        End If
      End If
    Catch ex As InvalidOperationException
      ' ignore if it fails...window too small, resize too early.
      ' All rare and can't do much about it.
    End Try
  End Sub

  Private Sub mnuAddFiles_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuAddFiles.Click
    ShowFileSelectionDialog()
  End Sub

  Private Sub mnuAddFolder_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuAddFolder.Click
    Using dialog As New FolderBrowserDialog
      dialog.ShowNewFolderButton = False

      Dim lastFolder As String = CStr(Settings.GetValue("Paths", "DataAddFolder", ""))
      If String.IsNullOrEmpty(lastFolder) Then
        dialog.SelectedPath = filSelect.CurrentFolder
      Else
        dialog.SelectedPath = lastFolder
      End If

      If dialog.ShowDialog(Me) = System.Windows.Forms.DialogResult.OK Then
        Settings.SetValue("Paths", "DataAddFolder", dialog.SelectedPath)
        AddToQueue(New String() {dialog.SelectedPath}, tvDirectories.GetSelectedPath())
      End If
      Exit Sub
    End Using
  End Sub

  Private Sub mnuUpdateCompilation_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuUpdateCompilation.Click
    m_FileLayoutManager.UpdateDataLayout()
  End Sub

  Private Sub mnuInvertSelection_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuInvertSelection.Click
    If lvFiles.Focused Then
      lvFiles.InvertSelection()
    Else
      filSelect.InvertSelection()
    End If
  End Sub

#Region "Disc file systems"

  Private Sub mnuFileSystem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuFileSystem.Click
    Using dialog As New FileSystemDialog
      dialog.DataImage = m_FileLayoutManager.DataImage
      If dialog.ShowDialog(Me) = DialogResult.OK Then
        m_FileLayoutManager.SetCompilationChanged(True)
      End If
    End Using
  End Sub

#End Region

  Private Sub m_FileLayoutManager_CompilationChanged(ByVal sender As System.Object, ByVal e As FileLayoutManager.CompilationChangedEventArgs) Handles m_FileLayoutManager.CompilationChanged
    Dim strExtraInfo As String = m_FileLayoutManager.LayoutName

    If m_FileLayoutManager.IsUnsaved Then
      strExtraInfo = strExtraInfo & "*"
    End If
    Me.Text = strExtraInfo & " - " & RM.GetString("Data Compilation")

    If e.ContentsChanged Then RecalcImageSize()

  End Sub

#Region "Context Menu"

  Private Sub cmnuProperties_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmuiProperties.Click
    m_FileLayoutManager.ShowProperties()
  End Sub

  Private Sub cmuCompilation_Popup(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmuCompilation.Popup
    ' Disable selections
    cmuiSelectAll.Visible = lvFiles.Focused
    cmuiInvertSelection.Visible = lvFiles.Focused
    cmuSep3.Visible = lvFiles.Focused

    cmuiPaste.Enabled = FileLayoutManager.HasClipboardData()

    ' Disable Edit
    cmuiRename.Enabled = (lvFiles.Focused AndAlso lvFiles.SelectedObject IsNot Nothing) OrElse (tvDirectories.Focused AndAlso tvDirectories.SelectedNode IsNot Nothing)
    cmuiRemove.Enabled = (lvFiles.Focused AndAlso lvFiles.SelectedIndices.Count > 0) OrElse (tvDirectories.Focused AndAlso tvDirectories.SelectedNode IsNot Nothing AndAlso tvDirectories.SelectedNode IsNot tvDirectories.Nodes(0))
    cmuiProperties.Enabled = cmuiRemove.Enabled
  End Sub

  Private Sub RenameSelection()
    If tvDirectories.Focused And Not tvDirectories.SelectedNode Is Nothing Then
      tvDirectories.SelectedNode.BeginEdit()
    Else
      lvFiles.EditSubItem(CType(lvFiles.FocusedItem, OLVListItem), 0)
    End If
  End Sub

  Private Sub DeleteSelection()
    If tvDirectories.Focused And Not tvDirectories.SelectedNode Is Nothing Then
      tvDirectories.DeleteSelection()
    Else
      lvFiles.DeleteSelection()
    End If
  End Sub

  Private Sub cmuiClear_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmuiClear.Click
    mnuClearCompilation.PerformClick()
  End Sub

  Private Sub cmuiRemove_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmuiRemove.Click
    DeleteSelection()
  End Sub

  Private Sub cmuiRename_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmuiRename.Click
    RenameSelection()
  End Sub

  Private Sub cmuiSelectAll_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmuiSelectAll.Click
    m_FileLayoutManager.ListView.SelectAll()
  End Sub

  Private Sub cmuiCreateDirectory_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmuiCreateDirectory.Click
    m_FileLayoutManager.CreateNewDirectory()
  End Sub

  Private Sub cmuiInvertSelection_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmuiInvertSelection.Click
    lvFiles.InvertSelection()
  End Sub

  Private Sub cmuiPaste_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmuiPaste.Click
    lvFiles.Paste()
  End Sub

  Private Sub cmuiUpdateCompilation_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmuiUpdateCompilation.Click
    m_FileLayoutManager.UpdateDataLayout()
  End Sub

#End Region

End Class