using System;
using System.Collections.Generic;
using System.Text;
using CDBurnerXP.Burning;
using System.Threading;
using System.Runtime.InteropServices;
using RocketDivision.StarBurnX;

namespace CDBurnerXP.CommandLine
{
    class Program
    {
        static int Main(string[] args)
        {
            CommandLineOptions options = null;

            ApplicationSettings.Load();

            // Parse options
            try
            {
                options = new CommandLineOptions(args);
            }
            catch (ArgumentNullException)
            {
                Console.WriteLine("No command line arguments given.");
                return 0;
            }

            if (options.Flags.Contains("-sptd"))
            {
                StarBurnX starBurn = new StarBurnX();
                byte major, minor;
                if (starBurn.CheckSPTDDriver(out major, out minor))
                {
                    BurningEngine.DeviceDriver = RocketDivision.StarBurnX.STARBURN_TRANSPORT.STARBURN_TRANSPORT_SPTD;
                }
                else
                {
                    Console.WriteLine("SPTD driver is not installed.");
                    return -1;
                }
            }

            // Execute what needs to be done
            try
            {
                switch (options.Command)
                {
                    case Command.ListDrives:
                        Commands.ListDrives();
                        break;

                    case Command.Erase:
                        Commands.Erase(options.DeviceIndex);
                        break;

                    case Command.Load:
                        Commands.Load(options.DeviceIndex);
                        break;

                    case Command.BurnData:
                        Commands.BurnData(options.DeviceIndex, options);
                        break;

                    case Command.BurnAudio:
                        Commands.BurnAudio(options.DeviceIndex, options);
                        break;

                    case Command.BurnIso:
                        Commands.BurnIso(options.DeviceIndex, options);
                        break;

                    case Command.WaitForDisc:
                        Commands.WaitForDisc(options.DeviceIndex);
                        break;

                    default:
                    case Command.Eject:
                        Commands.Eject(options.DeviceIndex);
                        break;
                }
            }
            catch (COMException ex)
            {
                Console.WriteLine("An error (" + ex.ErrorCode.ToString("X") + ") occured while executing the command: " + ex.Message);
                return ex.ErrorCode;
            }

#if DEBUG
            Thread.Sleep(10000);
#endif

            return 0;
        }
    }
}
