﻿using System;
using StarBurn = RocketDivision.StarBurnX;
using System.Runtime.InteropServices;

namespace CDBurnerXP.Burning
{
    /// <summary>
    /// AudioBurner with additional events.
    /// </summary>
    public class AudioBurner : StarBurn.AudioBurnerClass
    {
        public delegate void ProgressEventHandler(int tracknumber, bool isNextTrack, int totalPercent, TimeSpan timeRemaining);

        /// <summary>
        /// Fired on every status update of the burning process.
        /// </summary>
        public event ProgressEventHandler OnProgressExtended;

        private int m_TracksCompleted = 0;

        public AudioBurner()
        {
            this.OnProgress += new StarBurn.IAudioBurnerEvents_OnProgressEventHandler(AudioBurner_OnProgress);
        }

        public override void Burn(bool TestBurn)
        {
            m_TracksCompleted = 0;

            base.Burn(TestBurn);
        }

        private void AudioBurner_OnProgress(int percent, int timeRemaining)
        {
            bool isNextTrack = false;

            int totalProgress = 0;
            if (Mode == RocketDivision.StarBurnX.STARBURN_WRITE_MODE.STARBURN_WRITE_MODE_TAO)
            {
                totalProgress = (Tracks.Count == 0) ? 0 : Conversion.ToInt((m_TracksCompleted * 100.0 + percent) / Tracks.Count);
            }
            else
            {
                totalProgress = percent;
            }

            // isNextTrack = true for the very first track
            if (percent == 0 && m_TracksCompleted == 0)
            {
                isNextTrack = true;
            }
            else if (percent == 100 && m_TracksCompleted < Tracks.Count - 1)
            {
                // Increase track completed count only until we reach the last track
                m_TracksCompleted++;
                isNextTrack = true;
            }

            if (OnProgressExtended != null)
            {
                OnProgressExtended(m_TracksCompleted + 1, isNextTrack, totalProgress, TimeSpan.FromSeconds(timeRemaining));
            }
        }

        #region Useless overrides for COM

        public override RocketDivision.StarBurnX.AudioTrack AddFile(string FileName)
        {
            return base.AddFile(FileName);
        }

        public override string Artist
        {
            get
            {
                return base.Artist;
            }
            set
            {
                base.Artist = value;
            }
        }

        public override int LastWin32ErrorCode
        {
            get
            {
                return base.LastWin32ErrorCode;
            }
        }

        public override bool NextSessionAllowed
        {
            get
            {
                return base.NextSessionAllowed;
            }
            set
            {
                base.NextSessionAllowed = value;
            }
        }

        public override int AudioPauseLength
        {
            get
            {
                return base.AudioPauseLength;
            }
            set
            {
                base.AudioPauseLength = value;
            }
        }

        public override void Cancel()
        {
            base.Cancel();
        }

        public override RocketDivision.StarBurnX.Drive Drive
        {
            get
            {
                return base.Drive;
            }
            set
            {
                base.Drive = value;
            }
        }

        public override string LastError
        {
            get
            {
                return base.LastError;
            }
        }

        public override RocketDivision.StarBurnX.STARBURN_RESULT_CODES LastErrorCode
        {
            get
            {
                return base.LastErrorCode;
            }
        }

        public override bool LockTray
        {
            get
            {
                return base.LockTray;
            }
            set
            {
                base.LockTray = value;
            }
        }

        public override RocketDivision.StarBurnX.STARBURN_WRITE_MODE Mode
        {
            get
            {
                return base.Mode;
            }
            set
            {
                base.Mode = value;
            }
        }

        public override string Title
        {
            get
            {
                return base.Title;
            }
            set
            {
                base.Title = value;
            }
        }

        public override int WriteSpeed
        {
            get
            {
                return base.WriteSpeed;
            }
            set
            {
                try
                {
                    base.WriteSpeed = value;
                }
                catch (COMException ex)
                {
                    // Use default value, but note error
                    CDBurnerXP.IO.Debug.LogError(ex);
                }
            }
        }

        public override RocketDivision.StarBurnX.AudioTracks Tracks
        {
            get
            {
                return base.Tracks;
            }
        }

        public override bool IsXACDROM
        {
            get
            {
                return base.IsXACDROM;
            }
            set
            {
                base.IsXACDROM = value;
            }
        }

        public override bool KeepSessionOpened
        {
            get
            {
                return base.KeepSessionOpened;
            }
            set
            {
                base.KeepSessionOpened = value;
            }
        }

        public override bool CheckOverburning
        {
            get
            {
                return base.CheckOverburning;
            }
            set
            {
                base.CheckOverburning = value;
            }
        }

        #endregion

    }
}
