using System;
using System.Collections.Generic;
using System.Text;
using NMSDVDXLib;

namespace CDBurnerXP.Net
{
    internal class BurnProcessFailedException : Exception
    {
        private WriteErrorX m_Error = null;
        private string m_ErrorMessage = string.Empty;

        public override string Message
        {
            get
            {
                return m_ErrorMessage;
            }
        }

        public BurnProcessFailedException(WriteErrorX error)
        {
            m_Error = error;
            m_ErrorMessage = GenerateErrorMessage(m_Error);
        }

        protected string GenerateErrorMessage(WriteErrorX objError)
        {
            DeviceErrorX objDeviceError;
            FileErrorX objFileError;
            string strError = string.Empty;

            //Write errors are nested in the WriteError object
            //Get the correct error object for correct error code
            switch (objError.GetWriteErrorCode())
            {
                case eWriteErrorCodeX.weDeviceError:
                    //Get the device error
                    objDeviceError = objError.GetDeviceError();
                    strError = "(" + objDeviceError.GetDeviceErrorCode().ToString() + ")   " + objDeviceError.GetErrorContextMsg() + Environment.NewLine + objDeviceError.GetSenseErrorMsg() + " - " + objDeviceError.GetSenseString();
                    break;
                case eWriteErrorCodeX.weOutputFileError:
                    //Get the device error
                    objFileError = objError.GetFileError();
                    strError = "(" + objFileError.GetFileErrorCode().ToString() + ")   " + objFileError.GetErrorMsg() + Environment.NewLine + "'" + objFileError.GetFileName() + "'";
                    break;
                case eWriteErrorCodeX.weSourceFileError:
                    //Get the device error
                    objFileError = objError.GetFileError();
                    strError = "(" + objFileError.GetFileErrorCode().ToString() + ")   " + objFileError.GetErrorMsg() + Environment.NewLine + "'" + objFileError.GetFileName() + "'";
                    break;
                case eWriteErrorCodeX.weSystemError:
                    strError = "(" + objError.GetSystemErrorCode().ToString() + ")   " + objError.GetErrorMsg();
                    break;
                default:
                    //Other error codes are not asociated with objects
                    //So we will just display the error
                    strError = "(" + objError.GetWriteErrorCode().ToString() + ")   " + objError.GetErrorMsg();
                    break;
            }
            return strError;
        }


        public override string ToString()
        {
            return Message;
        }
    }
}
