﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using RocketDivision.StarBurnX;
using CDBurnerXP.Controls;

namespace CDBurnerXP.Forms
{
    /// <summary>
    /// Allows the user to set a custom disc type.
    /// </summary>
    public partial class CustomSpeedDialog : BaseForm
    {
        private BurnSpeed selectedSpeed;

        #region Properties

        /// <summary>
        /// Gets or sets the disc type to show burning speeds for.
        /// </summary>
        public STARBURN_DISC_TYPE DiscType { get; set; }

        /// <summary>
        /// Gets the user selected speed.
        /// </summary>
        public BurnSpeed SelectedSpeed
        {
            get
            {
                return this.selectedSpeed;
            }
        }

        protected override bool AutoTranslate
        {
            get
            {
                return true;
            }
        }

        #endregion

        public CustomSpeedDialog()
        {
            InitializeComponent();

            AcceptButton = bOK;
            CancelButton = bCancel;
        }

        protected override void OnLoad(EventArgs e)
        {
            base.OnLoad(e);

            List<BurnSpeed> customSpeeds = new List<BurnSpeed>();

            if ((this.DiscType & STARBURN_DISC_TYPE.STARBURN_DISC_TYPE_DVD) > 0)
            {
                for (int i = 20; i > 1; i -= 2)
                {
                    BurnSpeed speed = new BurnSpeed();
                    speed.SpeedShortValue = i;
                    speed.SpeedLongValue = CDBurnerXP.Burning.DiscType.DVDBaseSpeedInKBs * i;
                    speed.SpeedDescription = i + "x (" + speed.SpeedLongValue.ToString("0,0") + " KB/s)";
                    customSpeeds.Add(speed);
                }

                BurnSpeed speedSingle = new BurnSpeed();
                speedSingle.SpeedDescription = "1x";
                speedSingle.SpeedLongValue = CDBurnerXP.Burning.DiscType.DVDBaseSpeedInKBs;
                speedSingle.SpeedDescription = "1x (" + speedSingle.SpeedLongValue.ToString("0,0") + " KB/s)";
                customSpeeds.Add(speedSingle);
            }
            else if ((this.DiscType & STARBURN_DISC_TYPE.STARBURN_DISC_TYPE_BD) > 0)
            {
                for (int i = 8; i > 1; i -= 2)
                {
                    BurnSpeed speed = new BurnSpeed();
                    speed.SpeedShortValue = i;
                    speed.SpeedLongValue = CDBurnerXP.Burning.DiscType.BDBaseSpeedInKBs * i;
                    speed.SpeedDescription = i + "x (" + speed.SpeedLongValue.ToString("0,0") + " KB/s)";
                    customSpeeds.Add(speed);
                }

                BurnSpeed speedSingle = new BurnSpeed();
                speedSingle.SpeedDescription = "1x";
                speedSingle.SpeedLongValue = CDBurnerXP.Burning.DiscType.BDBaseSpeedInKBs;
                speedSingle.SpeedDescription = "1x (" + speedSingle.SpeedLongValue.ToString("0,0") + " KB/s)";
                customSpeeds.Add(speedSingle);
            }
            else // CD
            {
                for (int i = 52; i > 1; i -= 2)
                {
                    BurnSpeed speed = new BurnSpeed();
                    speed.SpeedShortValue = i;
                    speed.SpeedLongValue = (int)(CDBurnerXP.Burning.DiscType.CDBaseSpeedInKBs * i);
                    speed.SpeedDescription = i + "x (" + speed.SpeedLongValue.ToString("0,0") + " KB/s)";
                    customSpeeds.Add(speed);
                }

                BurnSpeed speedSingle = new BurnSpeed();
                speedSingle.SpeedDescription = "1x";
                speedSingle.SpeedLongValue = (int)CDBurnerXP.Burning.DiscType.CDBaseSpeedInKBs;
                speedSingle.SpeedDescription = "1x (" + speedSingle.SpeedLongValue.ToString("0,0") + " KB/s)";
                customSpeeds.Add(speedSingle);
            }

            lbSpeeds.DataSource = customSpeeds;
            lbSpeeds.ValueMember = "SpeedLongValue";
            lbSpeeds.DisplayMember = "SpeedDescription";
        }

        private void lbSpeeds_DoubleClick(object sender, EventArgs e)
        {
            bOK.PerformClick();
        }

        private void bOK_Click(object sender, EventArgs e)
        {
            this.selectedSpeed = (BurnSpeed)lbSpeeds.SelectedItem;
        }
    }
}
