﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using CDBurnerXP.Burning;
using CDBurnerXP.Controls;

namespace CDBurnerXP.Forms
{
    /// <summary>
    /// A dialof which shows all files that need to be renamed on the final disc
    /// due to file system limitations.
    /// </summary>
    public partial class FileNameLimitationsDialog : FileReportBase
    {
        private List<FileRenameRequirement> m_Errors = new List<FileRenameRequirement>();
        private FileLayoutManager manager = null;

        public FileRenameRequirement[] Errors
        {
            set
            {
                m_Errors.Clear();
                m_Errors.AddRange(value);
            }
            get
            {
                return m_Errors.ToArray();
            }
        }


        public FileNameLimitationsDialog()
        {
            InitializeComponent();

            AcceptButton = bOK;

            cName.Text = RM.GetString("Original name");
            cDetails.Text = RM.GetString("New name");
            cSourcePath.Text = RM.GetString("Path on disc");

            cName.AspectGetter = delegate(object x) { return ((FileRenameRequirement)x).originalName; };
            cName.ImageGetter = delegate(object x)
            {
                FileRenameRequirement requirement = x as FileRenameRequirement;
                return requirement.isFolder ? 3 : 0;
            };
            cDetails.AspectGetter = delegate(object x) { return ((FileRenameRequirement)x).newName; };
            cSourcePath.AspectGetter = delegate(object x) { return ((FileRenameRequirement)x).pathOnDisc; };
        }

        protected override void OnLoad(EventArgs e)
        {
            base.OnLoad(e);

            olvErrors.SetObjects(m_Errors);
        }

        /// <summary>
        /// Simply displays the dialog, for your convenience.
        /// </summary>
        /// <returns>true, if no errors were found or the user chose to ignore the errors</returns>
        public static bool Show(IWin32Window owner, FileRenameRequirement[] errors, FileLayoutManager manager)
        {
            // Don't show empty report
            if (errors == null || errors.Length == 0) return true;

            using (FileNameLimitationsDialog dialog = new FileNameLimitationsDialog())
            {
                dialog.Errors = errors;
                dialog.manager = manager;
                return (dialog.ShowDialog(owner) == DialogResult.OK);
            }
        }

        private void bChangeFileSystem_Click(object sender, EventArgs e)
        {
            // Allow user to make adjustments to the file system and check again
            using (FileSystemDialog dialog = new FileSystemDialog())
            {
                dialog.DataImage = manager.DataImage;
                if (dialog.ShowDialog(this) == System.Windows.Forms.DialogResult.OK)
                {
                    olvErrors.SetObjects(manager.CheckForFileNameChanges(this));
                    if (olvErrors.Objects.Count == 0)
                    {
                        bOK.Text = RM.GetString("Continue");
                    }
                }
            }
        }
    }
}
