﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using NMSDVDXLib;

namespace CDBurnerXP.Forms
{
    public partial class FileSystemDialog : BaseForm
    {
        #region LimitationInfo

        private class LimitationInfo
        {
            public string Limitation;
            public string Solution;

            public LimitationInfo(string limitation, string solution)
            {
                this.Limitation = limitation;
                this.Solution = solution;
            }
        }

        #endregion

        private IDataImageX dataImage;

        #region Properties

        protected override bool AutoTranslate
        {
            get
            {
                return true;
            }
        }

        public IDataImageX DataImage
        {
            get { return dataImage; }
            set { dataImage = value; }
        }

        #endregion

        public FileSystemDialog()
        {
            InitializeComponent();

            AcceptButton = bOK;
            CancelButton = bCancel;
        }

        protected override void OnLoad(EventArgs e)
        {
            base.OnLoad(e);

            colSolution.Text = RM.GetString("Solution");
            colLimitation.Text = RM.GetString("Limitation");
            olvRestrictions.EmptyListMsg = RM.GetString("No notable restrictions");

            switch (this.dataImage.VolumeType)
            {
                case eVolumeTypeX.vtISO9660: cboFileSystem.SelectedIndex = 0; break;
                case eVolumeTypeX.vtISO9660_Joliet: cboFileSystem.SelectedIndex = 1; break;
                case eVolumeTypeX.vtISO9660_Joliet_UDF: cboFileSystem.SelectedIndex = 2; break;
                case eVolumeTypeX.vtISO9660_UDF: cboFileSystem.SelectedIndex = 3; break;
                case eVolumeTypeX.vtUDF: cboFileSystem.SelectedIndex = 4; break;
            }

            switch (this.dataImage.ISOLevel)
            {
                case eISOLevelX.isoLevel01: cboIsoLevel.SelectedIndex = 0; break;
                case eISOLevelX.isoLevel02: cboIsoLevel.SelectedIndex = 1; break;
                case eISOLevelX.iso9660_1999: cboIsoLevel.SelectedIndex = 2; break;
            }

            chkLongJolietNames.Checked = this.dataImage.UseLongJolietNames;

            if (this.dataImage.IsImported())
            {
                chkLongJolietNames.Enabled = false;
                cboFileSystem.Enabled = false;
                lblIsoLevel.Enabled = false;
                lblFileSystem.Enabled = false;
                cboIsoLevel.Enabled = false;

                linkFileSystems.Text = RM.GetString("Why can't I change the file system?");
            }
        }

        private void cboFileSystem_SelectedIndexChanged(object sender, EventArgs e)
        {
            switch (cboFileSystem.SelectedIndex)
            {
                case 1:
                case 2:
                    cboIsoLevel.Enabled = true;
                    lblIsoLevel.Enabled = true;
                    chkLongJolietNames.Enabled = true;
                    break;
                case 4:
                    cboIsoLevel.Enabled = false;
                    lblIsoLevel.Enabled = false;
                    chkLongJolietNames.Enabled = false;
                    break;
                default:
                    cboIsoLevel.Enabled = true;
                    lblIsoLevel.Enabled = true;
                    chkLongJolietNames.Enabled = false;
                    break;
            }

            RefreshLimitations();
        }

        private void cboIsoLevel_SelectedIndexChanged(object sender, EventArgs e)
        {
            RefreshLimitations();
        }

        private void RefreshLimitations()
        {
            // Limitations we should mention:
            // - Only ISO without UDF/Joliet --> No Unicode support, limited file names [Solution: Use Joliet and/or UDF]
            // - Only ISO with 1999: Only recent OSes (>= WinXP) supported [Solution: Use UDF or lower ISO level]
            // - No UDF: File size limit of 4GB [Solution: Use UDF]

            List<LimitationInfo> limitations = new List<LimitationInfo>();
            if (cboFileSystem.SelectedIndex == 0)
            {
                limitations.Add(new LimitationInfo(RM.GetString("Only English characters (ASCII)"), RM.GetString("Use Joliet and/or UDF")));
            }
            if ((cboFileSystem.SelectedIndex == 0 || cboFileSystem.SelectedIndex == 1) && cboIsoLevel.SelectedIndex == 2)
            {
                limitations.Add(new LimitationInfo(RM.GetString("Only readable by recent OSes (WinXP and later)"), RM.GetString("Use UDF or ISO level 1/2")));
            }
            if (cboFileSystem.SelectedIndex == 0 || cboFileSystem.SelectedIndex == 1)
            {
                limitations.Add(new LimitationInfo(RM.GetString("No files larger than 4GiB (4294967296 bytes)"), RM.GetString("Use UDF")));
            }
            olvRestrictions.SetObjects(limitations);
        }

        private void bOK_Click(object sender, EventArgs e)
        {
            // Change data image settings

            switch (cboFileSystem.SelectedIndex)
            {
                case 0: this.dataImage.VolumeType = eVolumeTypeX.vtISO9660; break;
                case 1: this.dataImage.VolumeType = eVolumeTypeX.vtISO9660_Joliet; break;
                case 2: this.dataImage.VolumeType = eVolumeTypeX.vtISO9660_Joliet_UDF; break;
                case 3: this.dataImage.VolumeType = eVolumeTypeX.vtISO9660_UDF; break;
                case 4: this.dataImage.VolumeType = eVolumeTypeX.vtUDF; break;
            }

            switch (cboIsoLevel.SelectedIndex)
            {
                case 0: this.dataImage.ISOLevel = eISOLevelX.isoLevel01; break;
                case 1: this.dataImage.ISOLevel = eISOLevelX.isoLevel02; break;
                case 2: this.dataImage.ISOLevel = eISOLevelX.iso9660_1999; break;
            }

            this.dataImage.UseLongJolietNames = chkLongJolietNames.Checked;
        }
    }
}
