﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using System.IO;

namespace CDBurnerXP.Forms
{
    public class SafeOpenFileDialog
    {
        /// <summary>
        /// Shows an OpenFileDialog, but catches the InvalidOperationException.
        /// </summary>
        public static DialogResult ShowDialog(OpenFileDialog dialog, IWin32Window owner)
        {
            return ShowDialog(dialog, owner, null);
        }

        /// <summary>
        /// Shows an OpenFileDialog, but catches the InvalidOperationException.
        /// The InitialDirectory value is saved and restored, based on the savePathKey.
        /// </summary>
        public static DialogResult ShowDialog(OpenFileDialog dialog, IWin32Window owner, string savePathKey)
        {
            try
            {
                if (!String.IsNullOrEmpty(savePathKey))
                {
                    string previousPath = Settings.GetValue("Paths", savePathKey, "") as string;
                    if (!string.IsNullOrEmpty(previousPath))
                    {
                        try
                        {
                            dialog.InitialDirectory = Path.GetDirectoryName(previousPath);
                        }
                        catch (Exception)
                        {
                            // Use default path
                        }
                    }
                }

                DialogResult result = dialog.ShowDialog(owner);
                if (result == DialogResult.OK && !String.IsNullOrEmpty(savePathKey))
                {
                    Settings.SetValue("Paths", savePathKey, dialog.FileName);
                }

                return result;
            }
            catch (InvalidOperationException ex)
            {
                // User selected too many files. Show message and dialog again.
                MessageBox.Show(owner, ex.Message, Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Error);
                return ShowDialog(dialog, owner, savePathKey);
            }
        }
    }
}
