﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;

namespace CDBurnerXP.Forms
{
    public partial class SelectVideoFolderDialog : BaseForm
    {
        private string resultPath = string.Empty;

        #region Properties

        /// <summary>
        /// Returns the folder selected by the user.
        /// </summary>
        public string SelectedPath
        {
            get { return this.resultPath; }
        }

        protected override bool AutoTranslate
        {
            get
            {
                return true;
            }
        }

        #endregion

        public SelectVideoFolderDialog()
        {
            InitializeComponent();

            AcceptButton = bOK;
            CancelButton = bCancel;
        }

        protected override void OnLoad(EventArgs e)
        {
            base.OnLoad(e);

            lblCurrentFolder.Text = RM.GetString("Select a folder which contains valid video image files:");
            lblValidFiles.Text = RM.GetString("Valid files (.vob; .ifo; .bup):");
            fldVideoImage.FileView = flvVideoImage;
            flvVideoImage.FilePattern = "*.vob;*.ifo;*.bup";

            if (!string.IsNullOrEmpty(ApplicationSettings.DataVideoLayoutPath))
            {
                flvVideoImage.CurrentFolder = ApplicationSettings.DataVideoLayoutPath;
                flvVideoImage_AfterFill(this, null);
            }
        }

        protected override void OnClosing(CancelEventArgs e)
        {
            base.OnClosing(e);

            ApplicationSettings.DataVideoLayoutPath = flvVideoImage.CurrentFolder;
        }

        private void flvVideoImage_AfterFill(object sender, EventArgs e)
        {
            if (fldVideoImage.SelectedNode == null)
            {
                return;
            }

            int files = 0;
            try
            {
                string folderPath = fldVideoImage.SelectedNode.Path;

                foreach (string type in flvVideoImage.FilePattern.Split(';'))
                {
                    files += Directory.GetFiles(folderPath, type).Length;
                }

                // If files are located in a VIDEO_TS subfolder, show content of this folder instead
                string videoTSPath = Path.Combine(folderPath, "VIDEO_TS");
                if (files == 0 && Directory.Exists(videoTSPath))
                {
                    foreach (string type in flvVideoImage.FilePattern.Split(';'))
                    {
                        files += Directory.GetFiles(videoTSPath, type).Length;
                    }

                    if (files > 0)
                    {
                        flvVideoImage.CurrentFolder = videoTSPath;
                    }
                }
            }
            catch (UnauthorizedAccessException)
            {
                // no access allowed, ignore (Windows Vista will actually show its own dialog)
            }
            catch (IOException)
            {
                // no access or whatever
            }
            catch (ArgumentException)
            {
                // adding files from this path is not supported, ignore
            }

            bOK.Enabled = (files > 0);
        }

        private void bOK_Click(object sender, EventArgs e)
        {
            this.resultPath = flvVideoImage.CurrentFolder;
        }
    }
}
